@@ -16,6 +16,7 @@
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 from __future__ import annotations
 from typing import Dict, Union, TYPE_CHECKING, Tuple
+
 from GridCalEngine.basic_structures import Logger
 import GridCalEngine.Devices as dev
 from GridCalEngine.Devices.Substation.bus import Bus
@@ -29,6 +30,17 @@ import GridCalEngine.DataStructures as ds
 
 if TYPE_CHECKING:  # Only imports the below statements during type checking
     from GridCalEngine.Simulations.OPF.opf_results import OptimalPowerFlowResults
+    from GridCalEngine.Simulations.OPF.opf_ts_results import OptimalPowerFlowTimeSeriesResults
+    from GridCalEngine.Simulations.NTC.ntc_results import OptimalNetTransferCapacityResults
+    from GridCalEngine.Simulations.NTC.ntc_ts_results import OptimalNetTransferCapacityTimeSeriesResults
+    from GridCalEngine.Simulations import OptimalNetTransferCapacityResults
+
+    VALID_OPF_RESULTS = Union[
+        OptimalPowerFlowResults,
+        OptimalPowerFlowTimeSeriesResults,
+        OptimalNetTransferCapacityResults,
+        OptimalNetTransferCapacityTimeSeriesResults
+    ]
 
 
 def set_bus_control_voltage(i: int,
@@ -189,6 +201,12 @@ def get_load_data(circuit: MultiCircuit,
             data.active[ii] = elm.active
             data.cost[ii] = elm.Cost
 
+        # reactive power sharing data
+        if data.active[ii]:
+            bus_data.q_fixed[i] -= data.S[ii].imag
+            bus_data.ii_fixed[i] -= data.I[ii].imag
+            bus_data.b_fixed[i] -= data.Y[ii].imag
+
         data.C_bus_elm[i, ii] = 1
         ii += 1
 
@@ -209,6 +227,12 @@ def get_load_data(circuit: MultiCircuit,
             data.active[ii] = elm.active
             data.cost[ii] = elm.Cost
 
+        # reactive power sharing data
+        if data.active[ii]:
+            bus_data.q_fixed[i] += data.S[ii].imag
+            # bus_data.ii_fixed[i] += data.I[ii].imag
+            # bus_data.b_fixed[i] += data.Y[ii].imag
+
         data.C_bus_elm[i, ii] = 1
         ii += 1
 
@@ -243,6 +267,12 @@ def get_load_data(circuit: MultiCircuit,
             data.S[ii] += complex(elm.P, elm.Q)
             data.active[ii] = elm.active
 
+        # reactive power sharing data
+        if data.active[ii]:
+            bus_data.q_fixed[i] += data.S[ii].imag
+            bus_data.ii_fixed[i] += data.I[ii].imag
+            bus_data.b_fixed[i] += data.Y[ii].imag
+
         data.C_bus_elm[i, ii] = 1
         ii += 1
 
@@ -266,6 +296,12 @@ def get_load_data(circuit: MultiCircuit,
             data.active[ii] = elm.active
             data.cost[ii] = elm.Cost
 
+        # reactive power sharing data
+        if data.active[ii]:
+            bus_data.q_fixed[i] += data.S[ii].imag
+            bus_data.ii_fixed[i] += data.I[ii].imag
+            bus_data.b_fixed[i] += data.Y[ii].imag
+
         data.C_bus_elm[i, ii] = 1
         ii += 1
 
@@ -293,15 +329,15 @@ def get_shunt_data(
     :param t_idx:
     :param time_series:
     :param use_stored_guess:
+    :param control_remote_voltage:
     :return:
     """
-    devices = circuit.get_shunts()
 
     data = ds.ShuntData(nelm=circuit.get_shunt_like_device_number(),
                         nbus=circuit.get_bus_number())
 
     ii = 0
-    for k, elm in enumerate(devices):
+    for k, elm in enumerate(circuit.get_shunts()):
 
         i = bus_dict[elm.bus]
 
@@ -318,6 +354,10 @@ def get_shunt_data(
             data.active[k] = elm.active
             data.Y[k] = complex(elm.G, elm.B)
 
+        # reactive power sharing data
+        if data.active[ii]:
+            bus_data.b_fixed[i] += data.Y[ii].imag
+
         data.C_bus_elm[i, k] = 1
         ii += 1
 
@@ -382,6 +422,14 @@ def get_shunt_data(
                                         use_stored_guess=use_stored_guess,
                                         logger=logger)
 
+        # reactive power sharing data
+        if data.active[ii]:
+            if data.controllable[ii]:
+                bus_data.q_shared_total[i] += data.Y[ii].imag
+                data.q_share[ii] = data.Y[ii].imag
+            else:
+                bus_data.b_fixed[i] += data.Y[ii].imag
+
         data.C_bus_elm[i, ii] = 1
         ii += 1
 
@@ -394,7 +442,7 @@ def get_generator_data(
         bus_voltage_used: BoolVec,
         logger: Logger,
         bus_data: ds.BusData,
-        opf_results: Union[OptimalPowerFlowResults, None] = None,
+        opf_results: VALID_OPF_RESULTS | None = None,
         t_idx=-1,
         time_series=False,
         use_stored_guess=False,
@@ -416,8 +464,7 @@ def get_generator_data(
     """
     devices = circuit.get_generators()
 
-    data = ds.GeneratorData(nelm=len(devices),
-                            nbus=circuit.get_bus_number())
+    data = ds.GeneratorData(nelm=len(devices), nbus=circuit.get_bus_number())
 
     gen_index_dict: Dict[str, int] = dict()
     for k, elm in enumerate(devices):
@@ -536,6 +583,14 @@ def get_generator_data(
             data.qmin[k] = elm.Qmin
             data.qmax[k] = elm.Qmax
 
+        # reactive power sharing data
+        if data.active[k]:
+            if data.controllable[k]:
+                bus_data.q_shared_total[i] += data.p[k]
+                data.q_share[k] = data.p[k]
+            else:
+                bus_data.q_fixed[i] += data.get_q_at(k)
+
         data.C_bus_elm[i, k] = 1
 
     return data, gen_index_dict
@@ -547,7 +602,7 @@ def get_battery_data(
         bus_voltage_used: BoolVec,
         logger: Logger,
         bus_data: ds.BusData,
-        opf_results: Union[OptimalPowerFlowResults, None] = None,
+        opf_results: VALID_OPF_RESULTS | None = None,
         t_idx=-1,
         time_series=False,
         use_stored_guess=False,
@@ -696,6 +751,14 @@ def get_battery_data(
             data.qmin[k] = elm.Qmin
             data.qmax[k] = elm.Qmax
 
+        # reactive power sharing data
+        if data.active[k]:
+            if data.controllable[k]:
+                bus_data.q_shared_total[i] += data.p[k]
+                data.q_share[k] = data.p[k]
+            else:
+                bus_data.q_fixed[i] += data.get_q_at(k)
+
         data.C_bus_elm[i, k] = 1
 
     return data
@@ -792,7 +855,7 @@ def fill_controllable_branch(
         branch_tolerance_mode: BranchImpedanceMode,
         t_idx: int,
         time_series: bool,
-        opf_results: Union[OptimalPowerFlowResults, None],
+        opf_results: VALID_OPF_RESULTS | None,
         use_stored_guess: bool,
         bus_voltage_used: BoolVec,
         Sbase: float,
@@ -925,7 +988,7 @@ def get_branch_data(
         branch_tolerance_mode: BranchImpedanceMode,
         t_idx: int = -1,
         time_series: bool = False,
-        opf_results: Union[OptimalPowerFlowResults, None] = None,
+        opf_results: VALID_OPF_RESULTS | None = None,
         use_stored_guess: bool = False,
         control_taps_modules: bool = True,
         control_taps_phase: bool = True,
@@ -1148,7 +1211,7 @@ def get_hvdc_data(circuit: MultiCircuit,
                   bus_voltage_used: BoolVec,
                   t_idx=-1,
                   time_series=False,
-                  opf_results: Union[OptimalPowerFlowResults, None] = None,
+                  opf_results: Union[OptimalPowerFlowResults, OptimalNetTransferCapacityResults, None] = None,
                   use_stored_guess: bool = False,
                   logger: Logger = Logger()):
     """
