@@ -766,10 +766,10 @@ def add_transformer3w_data(circuit: MultiCircuit,
                                 V3=elm.V3,
                                 r12=elm.r12, r23=elm.r23, r31=elm.r31,
                                 x12=elm.x12, x23=elm.x23, x31=elm.x31,
-                                rate12=elm.rate12, rate23=elm.rate23, rate31=elm.rate31,
-                                contingency_rate12=elm.rate12,
-                                contingency_rate23=elm.rate23,
-                                contingency_rate31=elm.rate31, )
+                                rate12=elm.rate1, rate23=elm.rate2, rate31=elm.rate3,
+                                contingency_rate12=elm.rate1,
+                                contingency_rate23=elm.rate2,
+                                contingency_rate31=elm.rate3, )
 
         # this is because the central node is in the buses list already from GridCal
         tr3.central_node = bus_dict[elm.bus0.idtag]
@@ -1199,11 +1199,9 @@ def get_newton_pa_nonlinear_opf_options(pf_opt: PowerFlowOptions,
     q_control_dict = {False: npa.ReactivePowerControlMode.NoControl,
                       True: npa.ReactivePowerControlMode.Direct}
 
-    solver_dict = {MIPSolvers.CBC: npa.LpSolvers.Highs,
-                   MIPSolvers.HIGHS: npa.LpSolvers.Highs,
+    solver_dict = {MIPSolvers.HIGHS: npa.LpSolvers.Highs,
                    MIPSolvers.XPRESS: npa.LpSolvers.Xpress,
                    MIPSolvers.CPLEX: npa.LpSolvers.CPLEX,
-                   MIPSolvers.GLOP: npa.LpSolvers.Highs,
                    MIPSolvers.SCIP: npa.LpSolvers.Highs,
                    MIPSolvers.GUROBI: npa.LpSolvers.Gurobi}
 
@@ -1230,11 +1228,9 @@ def get_newton_pa_linear_opf_options(opf_opt: OptimalPowerFlowOptions,
     :param area_dict:
     :return:
     """
-    solver_dict = {MIPSolvers.CBC: npa.LpSolvers.Highs,
-                   MIPSolvers.HIGHS: npa.LpSolvers.Highs,
+    solver_dict = {MIPSolvers.HIGHS: npa.LpSolvers.Highs,
                    MIPSolvers.XPRESS: npa.LpSolvers.Xpress,
                    MIPSolvers.CPLEX: npa.LpSolvers.CPLEX,
-                   MIPSolvers.GLOP: npa.LpSolvers.Highs,
                    MIPSolvers.SCIP: npa.LpSolvers.Scip,
                    MIPSolvers.GUROBI: npa.LpSolvers.Gurobi}
 
@@ -1257,11 +1253,11 @@ def get_newton_pa_linear_opf_options(opf_opt: OptimalPowerFlowOptions,
     opt.use_ramp_constraints = False
     opt.lodf_threshold = opf_opt.lodf_tolerance
 
-    if opf_opt.aggregations_from is not None:
-        opt.areas_from = [area_dict[e] for e in opf_opt.aggregations_from]
-
-    if opf_opt.aggregations_to is not None:
-        opt.areas_to = [area_dict[e] for e in opf_opt.aggregations_to]
+    # if opf_opt.aggregations_from is not None:
+    #     opt.areas_from = [area_dict[e] for e in opf_opt.aggregations_from]
+    #
+    # if opf_opt.aggregations_to is not None:
+    #     opt.areas_to = [area_dict[e] for e in opf_opt.aggregations_to]
 
     return opt
 
@@ -1518,9 +1514,15 @@ def translate_newton_pa_pf_results(grid: MultiCircuit, res: "npa.PowerFlowResult
     results = PowerFlowResults(n=grid.get_bus_number(),
                                m=grid.get_branch_number_wo_hvdc(),
                                n_hvdc=grid.get_hvdc_number(),
+                               n_gen=grid.get_generators_number(),
+                               n_batt=grid.get_batteries_number(),
+                               n_sh=grid.get_shunt_like_device_number(),
                                bus_names=res.bus_names,
                                branch_names=res.branch_names,
                                hvdc_names=res.hvdc_names,
+                               gen_names=grid.get_generator_names(),
+                               batt_names=grid.get_battery_names(),
+                               sh_names=grid.get_shunt_like_devices_names(),
                                bus_types=res.bus_types)
 
     results.voltage = res.voltage[0, :]
