@@ -17,6 +17,8 @@
 
 from GridCalEngine.Devices.Aggregation.contingency import Contingency
 from GridCalEngine.Devices.Aggregation.contingency_group import ContingencyGroup
+from GridCalEngine.Devices.Aggregation.remedial_action import RemedialAction
+from GridCalEngine.Devices.Aggregation.remedial_action_group import RemedialActionGroup
 from GridCalEngine.Devices.Aggregation.investment import Investment
 from GridCalEngine.Devices.Aggregation.investments_group import InvestmentsGroup
 from GridCalEngine.Devices.Aggregation.area import Area
@@ -28,3 +30,4 @@ from GridCalEngine.Devices.Aggregation.municipality import Municipality
 from GridCalEngine.Devices.Aggregation.branch_group import BranchGroup
 from GridCalEngine.Devices.Aggregation.modelling_authority import ModellingAuthority
 from GridCalEngine.Devices.Aggregation.inter_aggregation_info import InterAggregationInfo
+from GridCalEngine.Devices.Aggregation.facility import Facility
