@@ -88,7 +88,7 @@ class Associations:
         self._device_type = device_type
 
     @property
-    def data(self):
+    def data(self) -> Dict[str, Association]:
         """
 
         :return:
@@ -168,6 +168,13 @@ class Associations:
         """
         return [val.to_dict() for _, val in self._data.items()]
 
+    def to_list(self) -> List[ALL_DEV_TYPES]:
+        """
+        Get a list of the associated api objects
+        :return:
+        """
+        return [val.api_object for _, val in self._data.items()]
+
     def parse(self,
               data: List[Dict[str, Union[str, float]]],
               elements_dict: Dict[str, ALL_DEV_TYPES],
