@@ -109,7 +109,7 @@ class DcLine(BranchParent):
         self.measurements = list()
 
         # line length in km
-        self.length = float(length)
+        self._length = float(length)
 
         # line impedance tolerance
         self.tolerance = float(tolerance)
@@ -142,7 +142,7 @@ class DcLine(BranchParent):
         self.register(key='fault_pos', units='p.u.', tpe=float,
                       definition='Per-unit positioning of the fault:0 would be at the "from" side,1 would '
                                  'be at the "to" side,therefore 0.5 is at the middle.')
-        self.register(key='template', units='', tpe=DeviceType.SequenceLineDevice, definition='', editable=False)
+        self.register(key='template', units='', tpe=DeviceType.AnyLineTemplateDevice, definition='', editable=False)
 
         self.register(key='locations', units='', tpe=SubObjectType.LineLocations, definition='', editable=False)
 
@@ -190,13 +190,45 @@ class DcLine(BranchParent):
         """
         return self.R * (1 + self.alpha * (self.temp_oper - self.temp_base))
 
-    def change_base(self, Sbase_old, Sbase_new):
+    @property
+    def length(self) -> float:
+        """
+        Line length in km
+        :return: float
+        """
+        return self._length
 
+    @length.setter
+    def length(self, val: float):
+        if isinstance(val, float):
+            if val > 0.0:
+
+                if self._length != 0:
+                    factor = np.round(val / self._length, 6)  # new length / old length
+
+                    self.R *= factor
+
+                self._length = val
+            else:
+                print('The length cannot be zero, ignoring value')
+        else:
+            raise Exception('The length must be a float value')
+
+    def change_base(self, Sbase_old, Sbase_new):
+        """
+
+        :param Sbase_old:
+        :param Sbase_new:
+        """
         b = Sbase_new / Sbase_old
 
         self.R *= b
 
     def get_weight(self):
+        """
+
+        :return:
+        """
         return self.R
 
     def copy(self, bus_dict=None):
