@@ -90,6 +90,7 @@ class Transformer2W(ControllableBranchParent):
                  build_status: BuildStatus = BuildStatus.Commissioned,
                  tc_total_positions: int = 5,
                  tc_neutral_position: int = 2,
+                 tc_normal_position: int = 2,
                  tc_dV: float = 0.01,
                  tc_asymmetry_angle=90,
                  tc_type: TapChangerTypes = TapChangerTypes.NoRegulation):
@@ -200,6 +201,7 @@ class Transformer2W(ControllableBranchParent):
                                           device_type=DeviceType.Transformer2WDevice,
                                           tc_total_positions=tc_total_positions,
                                           tc_neutral_position=tc_neutral_position,
+                                          tc_normal_position=tc_normal_position,
                                           tc_dV=tc_dV,
                                           tc_asymmetry_angle=tc_asymmetry_angle,
                                           tc_type=tc_type)
