@@ -82,6 +82,7 @@ class ControllableBranchParent(BranchParent):
                  device_type: DeviceType,
                  tc_total_positions: int = 5,
                  tc_neutral_position: int = 2,
+                 tc_normal_position: int = 2,
                  tc_dV: float = 0.01,
                  tc_asymmetry_angle=90,
                  tc_type: TapChangerTypes = TapChangerTypes.NoRegulation):
@@ -186,6 +187,7 @@ class ControllableBranchParent(BranchParent):
         # tap changer object
         self._tap_changer = TapChanger(total_positions=tc_total_positions,
                                        neutral_position=tc_neutral_position,
+                                       normal_position=tc_normal_position,
                                        dV=tc_dV,
                                        asymmetry_angle=tc_asymmetry_angle,
                                        tc_type=tc_type)
