@@ -24,7 +24,7 @@ from GridCalEngine.enumerations import (DeviceType, TimeFrame, BuildStatus, Wind
                                         HvdcControlType, ActionType, AvailableTransferMode, ContingencyMethod,
                                         CpfParametrization, CpfStopAt, InvestmentEvaluationMethod, SolverType,
                                         InvestmentsEvaluationObjectives, NodalCapacityMethod, TimeGrouping,
-                                        ZonalGrouping, MIPSolvers, AcOpfMode,
+                                        ZonalGrouping, MIPSolvers, AcOpfMode, SubstationTypes,
                                         BranchImpedanceMode, FaultType, TapChangerTypes)
 
 # types that can be assigned to a GridCal property
@@ -55,7 +55,8 @@ GCPROP_TYPES = Union[
     Type[AcOpfMode],
     Type[BranchImpedanceMode],
     Type[FaultType],
-    Type[TapChangerTypes]
+    Type[TapChangerTypes],
+    Type[SubstationTypes]
 ]
 
 
@@ -763,7 +764,7 @@ class EditableDevice:
         """
         Create a deep copy of this object
         """
-        tpe = type(self)
+        tpe = self.__class__
 
         try:
             new_obj = tpe(name=self.name,
@@ -771,13 +772,14 @@ class EditableDevice:
                           code=self.code,
                           device_type=self.device_type)
         except TypeError:
-            new_obj = tpe(name=self.name,
-                          idtag=uuid.uuid4().hex if forced_new_idtag else self.idtag,
-                          code=self.code)
+            new_obj = tpe()
 
         for prop_name, value in self.__dict__.items():
             setattr(new_obj, prop_name, value)
 
+        if forced_new_idtag:
+            new_obj.idtag = uuid.uuid4().hex
+
         return new_obj
 
     @staticmethod
