@@ -19,6 +19,8 @@ from typing import Union, Dict, Tuple, List, Any
 from collections import Counter
 import numpy as np
 import numba as nb
+from numpy import dtype
+
 from GridCalEngine.basic_structures import Numeric, NumericVec, IntVec
 from GridCalEngine.enumerations import DeviceType
 from GridCalEngine.Utils.Sparse.sparse_array import SparseArray, PROFILE_TYPES, check_type
@@ -255,7 +257,7 @@ class Profile:
         else:
             return 0.0
 
-    def set(self, arr: NumericVec):
+    def set(self, arr: NumericVec) -> bool:
         """
         Set array value
         :param arr:
@@ -278,31 +280,38 @@ class Profile:
                 if isinstance(base, np.bool_):
                     base = bool(base)
 
-                self._is_sparse = True
-                self._sparse_array = SparseArray(data_type=self.dtype)
+                if check_type(dtype=self.dtype, value=base):
+                    self._is_sparse = True
+                    self._sparse_array = SparseArray(data_type=self.dtype)
 
-                if most_common_count > 1:
-                    if isinstance(arr, np.ndarray):
-                        data, indptr = compress_array_numba(arr, base)
-                        data_map = {i: x for i, x in zip(indptr, data)}  # this is to use a native python dict
+                    if most_common_count > 1:
+                        if isinstance(arr, np.ndarray):
+                            data, indptr = compress_array_numba(arr, base)
+                            data_map = {i: x for i, x in zip(indptr, data)}  # this is to use a native python dict
+                        else:
+                            raise Exception('Unknown profile type' + str(type(arr)))
                     else:
-                        raise Exception('Unknown profile type' + str(type(arr)))
-                else:
-                    data_map = dict()
+                        data_map = dict()
 
-                self._sparse_array.create(size=len(arr),
-                                          default_value=base,
-                                          data=data_map)
-                check_type(dtype=self.dtype, value=base)
+                    self._sparse_array.create(size=len(arr),
+                                              default_value=base,
+                                              data=data_map)
+                else:
+                    print("Cannot set sparse array because the type check failed")
+                    return False
             else:
-                check_type(dtype=self.dtype, value=arr[0])
-                self._is_sparse = False
-                self._dense_array = arr
+                if check_type(dtype=self.dtype, value=arr[0]):
+                    self._is_sparse = False
+                    self._dense_array = arr
+                else:
+                    print("Cannot set dense array because the type check failed")
+                    return False
         else:
             self._is_sparse = False
             self._dense_array = arr
 
         self._initialized = True
+        return True
 
     def __eq__(self, other: "Profile") -> bool:
         """
@@ -329,7 +338,16 @@ class Profile:
         if self._is_sparse:
             return self._sparse_array[key]
         else:
-            return self._dense_array[key]
+
+            if self._dense_array is None:
+                # WTF, initialize sparse
+                self._is_sparse = True
+                self._sparse_array = SparseArray(data_type=self.dtype)
+                self._sparse_array.default_value = self.default_value
+                print("Initializing sparse when querying, this signals a mis initilaization")
+                return self.default_value
+            else:
+                return self._dense_array[key]
 
     def __setitem__(self, key: int, value):
         """
@@ -385,7 +403,12 @@ class Profile:
                 if self._is_sparse:
                     self._sparse_array.resize(n=n)
                 else:
-                    self._dense_array.resize(n)
+                    try:
+                        self._dense_array.resize(n)
+                    except ValueError:
+                        new_arr = np.zeros(n, dtype=self._dense_array.dtype)
+                        new_arr[:len(self._dense_array)] = self._dense_array
+                        self._dense_array = new_arr  # this is to avoid ValueError when resizing a numpy array of Objects
             else:
                 self._initialized = True
                 self.create_sparse(size=n, default_value=self.default_value)
