@@ -14,7 +14,6 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-from typing import Union
 from GridCalEngine.Devices.Aggregation import *
 from GridCalEngine.Devices.Associations import *
 from GridCalEngine.Devices.Branches import *
@@ -108,11 +107,14 @@ ALL_DEV_TYPES = Union[
     LineLocations,
     LineLocation,
     ModellingAuthority,
+    Facility,
     Fuel,
     Investment,
     InvestmentsGroup,
     Contingency,
     ContingencyGroup,
+    RemedialAction,
+    RemedialActionGroup,
     Technology,
     UndergroundLineType,
     SequenceLineType
@@ -120,5 +122,4 @@ ALL_DEV_TYPES = Union[
 
 CONNECTION_TYPE = Union[ConnectivityNode, Bus, None]
 
-
 ASSOCIATION_TYPES = Union[Fuel, Technology, EmissionGas]
