@@ -20,11 +20,11 @@ from GridCalEngine.IO.cim.cgmes.cgmes_enums import cgmesProfile
 
 
 class SvTapStep(Base):
-	def __init__(self, rdfid, tpe, resources=list(), class_replacements=dict()):
+	def __init__(self, rdfid, tpe='SvTapStep', resources=list(), class_replacements=dict()):
 		Base.__init__(self, rdfid=rdfid, tpe=tpe, resources=resources, class_replacements=class_replacements)
 
 		self.position: float = None
-		from GridCalEngine.IO.cim.cgmes.cgmes_v3_0_0.devices.tap_changer import TapChanger
+		from GridCalEngine.IO.cim.cgmes.cgmes_v2_4_15.devices.tap_changer import TapChanger
 		self.TapChanger: TapChanger | None = None
 
 		self.register_property(
@@ -32,7 +32,7 @@ class SvTapStep(Base):
 			class_type=float,
 			multiplier=UnitMultiplier.none,
 			unit=UnitSymbol.none,
-			description='''The floating point tap position.   This is not the tap ratio, but rather the tap step position as defined by the related tap changer model and normally is constrained to be within the range of minimum and maximum tap positions.''',
+			description='''A floating point number. The range is unspecified and not limited.''',
 			profiles=[]
 		)
 		self.register_property(
