@@ -15,7 +15,7 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 from GridCalEngine.IO.base.units import UnitMultiplier, UnitSymbol
-from GridCalEngine.IO.cim.cgmes.cgmes_v2_4_15.devices.operational_limit import OperationalLimit
+from GridCalEngine.IO.cim.cgmes.cgmes_v3_0_0.devices.operational_limit import OperationalLimit
 from GridCalEngine.IO.cim.cgmes.cgmes_enums import cgmesProfile, UnitSymbol
 
 
@@ -23,8 +23,17 @@ class ActivePowerLimit(OperationalLimit):
 	def __init__(self, rdfid='', tpe='ActivePowerLimit'):
 		OperationalLimit.__init__(self, rdfid, tpe)
 
+		self.normalValue: float = None
 		self.value: float = None
 
+		self.register_property(
+			name='normalValue',
+			class_type=float,
+			multiplier=UnitMultiplier.M,
+			unit=UnitSymbol.W,
+			description='''Product of RMS value of the voltage and the RMS value of the in-phase component of the current.''',
+			profiles=[]
+		)
 		self.register_property(
 			name='value',
 			class_type=float,
