@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """
 GridCal
-# Copyright (C) 2015 - 2023 Santiago Peñate Vera
+# Copyright (C) 2015 - 2024 Santiago Peñate Vera
 # 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -18,8 +18,8 @@ GridCal
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 """
 
-from GridCalEngine.Core.Devices.multi_circuit import MultiCircuit
-import GridCalEngine.Core.Devices as dev
+from GridCalEngine.Devices.multi_circuit import MultiCircuit
+import GridCalEngine.Devices as dev
 import math
 import numpy as np
 from numpy import array
@@ -633,7 +633,7 @@ def data_to_grid_object(data, pos_dict, codification="utf-8") -> MultiCircuit:
         buses_dict[ID] = i
         bus_name = buses['loc_name'][i].decode(codification)  # BUS_Name
         vnom = buses['uknom'][i]
-        bus = dev.Bus(name=bus_name, vnom=vnom, vmin=0.9, vmax=1.1, xpos=x, ypos=-y, active=True)
+        bus = dev.Bus(name=bus_name, Vnom=vnom, vmin=0.9, vmax=1.1, xpos=x, ypos=-y, active=True)
         circuit.add_bus(bus)
 
     ####################################################################################################################
@@ -683,21 +683,6 @@ def data_to_grid_object(data, pos_dict, codification="utf-8") -> MultiCircuit:
             # create the slack entry on buses
             bus_obj.is_slack = True
 
-            # BUSES[bus1, bd.BUS_TYPE] = 3
-            # BUSES[bus1, bd.VA] = va
-            # BUSES[bus1, bd.VM] = vm
-            #
-            # # create the slack entry on generators (add the slack generator)
-            # gen_ = gen_line.copy()
-            # gen_[gd.GEN_BUS] = bus1
-            # gen_[gd.MBASE] = baseMVA
-            # gen_[gd.VG] = vm
-            # gen_[gd.GEN_STATUS] = 1
-            # gen_[gd.PG] += external['pgini'].values[i]
-            #
-            # GEN.append(gen_)
-            # GEN_NAMES.append(external['loc_name'][i])
-
         elif external['bustp'].values[i] == b'PV':
 
             if 'pgini' in external.columns.values:
@@ -708,8 +693,7 @@ def data_to_grid_object(data, pos_dict, codification="utf-8") -> MultiCircuit:
             # add a generator to the bus
             gen = dev.Generator(name=external['loc_name'][i].decode(codification),
                                 P=p,
-                                vset=vm, Qmin=-9999, Qmax=9999, Snom=9999,
-                                P_prof=None, vset_prof=None)
+                                vset=vm, Qmin=-9999, Qmax=9999, Snom=9999)
             circuit.add_generator(bus_obj, gen)
 
             # # mark the bus as pv
@@ -795,18 +779,15 @@ def data_to_grid_object(data, pos_dict, codification="utf-8") -> MultiCircuit:
             Irated = np.double(lines_rate[type_idx])  # kA
             Smax = Irated * vbase  # MVA
 
-            line = dev.Branch(bus_from=bus_from, bus_to=bus_to,
-                              name=lines['loc_name'][i].decode(codification),
-                              r=r,
-                              x=l,
-                              g=1e-20,
-                              b=b,
-                              rate=Smax,
-                              tap=1,
-                              shift_angle=0,
-                              active=status, mttf=0, mttr=0)
+            line = dev.Line(bus_from=bus_from, bus_to=bus_to,
+                            name=lines['loc_name'][i].decode(codification),
+                            r=r,
+                            x=l,
+                            b=b,
+                            rate=Smax,
+                            active=status, mttf=0, mttr=0)
 
-            circuit.add_branch(line)
+            circuit.add_line(line)
 
             # # put all in the correct column
             # line_[brd.F_BUS] = bus1
@@ -929,22 +910,21 @@ def data_to_grid_object(data, pos_dict, codification="utf-8") -> MultiCircuit:
 
                 status = 1 - transformers['outserv'][i]
 
-                trafo = dev.Branch(bus_from=bus_from,
-                                   bus_to=bus_to,
-                                   name=transformers['loc_name'][i].decode(codification),
-                                   r=Zs.real,
-                                   x=Zs.imag,
-                                   g=Ysh.real,
-                                   b=Ysh.imag,
-                                   rate=Smax,
-                                   tap=1.0,
-                                   shift_angle=0.0,
-                                   active=status,
-                                   mttf=0,
-                                   mttr=0,
-                                   branch_type=dev.BranchType.Transformer)
+                trafo = dev.Transformer2W(bus_from=bus_from,
+                                          bus_to=bus_to,
+                                          name=transformers['loc_name'][i].decode(codification),
+                                          r=Zs.real,
+                                          x=Zs.imag,
+                                          g=Ysh.real,
+                                          b=Ysh.imag,
+                                          rate=Smax,
+                                          tap_module=1.0,
+                                          tap_phase=0.0,
+                                          active=status,
+                                          mttf=0,
+                                          mttr=0, )
 
-                circuit.add_branch(trafo)
+                circuit.add_transformer2w(trafo)
 
             else:
                 warn('Transformer type not found!')
@@ -1121,9 +1101,7 @@ def data_to_grid_object(data, pos_dict, codification="utf-8") -> MultiCircuit:
                             vset=vnom,
                             Qmin=synchronous_machine['q_min'][i] * num_machines * snom,
                             Qmax=synchronous_machine['q_max'][i] * num_machines * snom,
-                            Snom=snom,
-                            P_prof=None,
-                            vset_prof=None)
+                            Snom=snom)
         circuit.add_generator(bus_obj, gen)
 
         # if synchronous_machine['pgini'][i] != 0:
