@@ -51,6 +51,7 @@ from GridCalEngine.Devices.multi_circuit import MultiCircuit
 from GridCalEngine.Simulations.results_template import DriverToSave
 from GridCalEngine.Simulations.PowerFlow.power_flow_results import PowerFlowResults
 from GridCalEngine.enumerations import CGMESVersions, SimulationTypes
+from GridCalEngine.DataStructures.numerical_circuit import compile_numerical_circuit_at
 
 if TYPE_CHECKING:
     from GridCalEngine.Simulations.types import DRIVER_OBJECTS
@@ -68,7 +69,9 @@ class FileSavingOptions:
                  dictionary_of_json_files: Dict[str, Dict[str, Any]] = None,
                  cgmes_version: CGMESVersions = CGMESVersions.v2_4_15,
                  cgmes_profiles: Union[None, List[cgmesProfile]] = None,
-                 one_file_per_profile: bool = False):
+                 cgmes_one_file_per_profile: bool = False,
+                 cgmes_map_areas_like_raw: bool = False,
+                 raw_version: str = "33"):
         """
         Constructor
         :param cgmes_boundary_set: CGMES boundary set zip file path
@@ -77,7 +80,9 @@ class FileSavingOptions:
         :param dictionary_of_json_files: Dictionary of json files
         :param cgmes_version: Version to use with CGMES
         :param cgmes_profiles: CGMES profile list to export
-        :param one_file_per_profile: use one file per profile?
+        :param cgmes_one_file_per_profile: use one file per profile?
+        :param cgmes_map_areas_like_raw: use map areas like raw?
+        :param raw_version: Version to use when exporting raw/rawx files
         """
 
         self.cgmes_version: CGMESVersions = cgmes_version
@@ -105,7 +110,11 @@ class FileSavingOptions:
                                                                                  cgmesProfile.GL]
 
         # use one file per profile?
-        self.one_file_per_profile = one_file_per_profile
+        self.cgmes_one_file_per_profile = cgmes_one_file_per_profile
+
+        self.cgmes_map_areas_like_raw = cgmes_map_areas_like_raw
+
+        self.raw_version = raw_version
 
     def get_power_flow_results(self) -> Union[None, PowerFlowResults]:
         """
@@ -119,6 +128,16 @@ class FileSavingOptions:
         return None
 
 
+class FileOpenOptions:
+    """
+    This class is to store the extra stuff that needs to be passed to open more complex files
+    """
+
+    def __init__(self,
+                 cgmes_map_areas_like_raw: bool = False):
+        self.cgmes_map_areas_like_raw = cgmes_map_areas_like_raw
+
+
 class FileOpen:
     """
     File open interface
@@ -126,16 +145,20 @@ class FileOpen:
 
     def __init__(self,
                  file_name: Union[str, List[str]],
-                 previous_circuit: Union[MultiCircuit, None] = None):
+                 previous_circuit: Union[MultiCircuit, None] = None,
+                 options: FileOpenOptions | None = None):
         """
         File open handler
         :param file_name: name of the file
         :param previous_circuit: previous circuit
+        :param options: FileOpenOptions
         """
         self.file_name: Union[str, List[str]] = file_name
 
         self.circuit: Union[MultiCircuit, None] = None
 
+        self.options: FileOpenOptions = options if options is not None else FileOpenOptions()
+
         self.cgmes_circuit: Union[CgmesCircuit, None] = None
 
         self.json_files = dict()
@@ -177,6 +200,7 @@ class FileOpen:
             data_parser = CgmesDataParser(text_func=text_func, progress_func=progress_func, logger=self.cgmes_logger)
             data_parser.load_files(files=self.file_name)
             self.cgmes_circuit = CgmesCircuit(cgmes_version=data_parser.cgmes_version, text_func=text_func,
+                                              cgmes_map_areas_like_raw=self.options.cgmes_map_areas_like_raw,
                                               progress_func=progress_func, logger=self.cgmes_logger)
             self.cgmes_circuit.parse_files(data_parser=data_parser)
             self.circuit = cgmes_to_gridcal(cgmes_model=self.cgmes_circuit, logger=self.cgmes_logger)
@@ -334,6 +358,7 @@ class FileOpen:
 
                     if is_valid_cgmes(data_parser.cgmes_version):
                         self.cgmes_circuit = CgmesCircuit(cgmes_version=data_parser.cgmes_version, text_func=text_func,
+                                                          cgmes_map_areas_like_raw=self.options.cgmes_map_areas_like_raw,
                                                           progress_func=progress_func, logger=self.cgmes_logger)
                         self.cgmes_circuit.parse_files(data_parser=data_parser)
                         self.circuit = cgmes_to_gridcal(cgmes_model=self.cgmes_circuit, logger=self.cgmes_logger)
@@ -542,9 +567,12 @@ class FileSave:
         data_parser.load_files(files=[self.options.cgmes_boundary_set])
         cgmes_circuit.parse_files(data_parser=data_parser)
         profiles_to_export = self.options.cgmes_profiles
-        one_file_per_profile = self.options.one_file_per_profile
+        one_file_per_profile = self.options.cgmes_one_file_per_profile
+        nc = compile_numerical_circuit_at(self.circuit)
         pf_results = self.options.get_power_flow_results()
+
         cgmes_circuit = gridcal_to_cgmes(gc_model=self.circuit,
+                                         num_circ=nc,
                                          cgmes_model=cgmes_circuit,
                                          pf_results=pf_results,
                                          logger=logger)
