@@ -58,6 +58,8 @@ def get_objects_dictionary() -> Dict[str, ALL_DEV_TYPES]:
 
         'emission': dev.EmissionGas(),
 
+        'facility': dev.Facility(),
+
         'bus': dev.Bus(),
 
         'bus_bar': dev.BusBar(),
@@ -95,12 +97,12 @@ def get_objects_dictionary() -> Dict[str, ALL_DEV_TYPES]:
         'transformer3w': dev.Transformer3W(),
 
         'line': dev.Line(),
-        'dc_line': dev.DcLine(None, None),
+        'dc_line': dev.DcLine(),
 
         'hvdc': dev.HvdcLine(),
 
-        'vsc': dev.VSC(None, None),
-        'upfc': dev.UPFC(None, None),
+        'vsc': dev.VSC(),
+        'upfc': dev.UPFC(),
 
         'series_reactance': dev.SeriesReactance(),
 
@@ -109,14 +111,12 @@ def get_objects_dictionary() -> Dict[str, ALL_DEV_TYPES]:
         'contingency_group': dev.ContingencyGroup(),
         'contingency': dev.Contingency(),
 
+        'remedial_action_group': dev.RemedialActionGroup(),
+        'remedial_action': dev.RemedialAction(),
+
         'investments_group': dev.InvestmentsGroup(),
         'investment': dev.Investment(),
 
-        # TODO: Handle these legacy types
-        # 'generator_technology': dev.GeneratorTechnology(),
-        # 'generator_fuel': dev.GeneratorFuel(),
-        # 'generator_emission': dev.GeneratorEmission(),
-
         'fluid_node': dev.FluidNode(),
         'fluid_path': dev.FluidPath(),
         'fluid_turbine': dev.FluidTurbine(),
@@ -267,8 +267,7 @@ def profile_todict(profile: Profile) -> Dict[str, str]:
         return {
             'is_sparse': True,
             'size': s,
-            'default': profile.default_value
-            if profile.sparse_array is None else profile.sparse_array.default_value,
+            'default': profile.default_value if profile.sparse_array is None else profile.sparse_array.default_value,
             'sparse_data': {
                 'map': dict()
             }
@@ -288,8 +287,9 @@ def profile_todict_idtag(profile: Profile) -> Dict[str, str]:
             'size': profile.size(),
             'default': default,
             'sparse_data': {
-                'map': {key: val.idtag for key, val in profile.sparse_array.get_map().items()}
-                if profile.sparse_array else dict()
+                'map': {key: val.idtag if hasattr(val, 'idtag') else None
+                        for key, val in profile.sparse_array.get_map().items()}
+                if profile.sparse_array is not None else dict()
             }
         }
     else:
@@ -297,7 +297,8 @@ def profile_todict_idtag(profile: Profile) -> Dict[str, str]:
             'is_sparse': profile.is_sparse,
             'size': profile.size(),
             'default': default,
-            'dense_data': [e.idtag for e in profile.dense_array] if profile.dense_array else list(),
+            'dense_data': [e.idtag if hasattr(e, 'idtag') else None for e in profile.dense_array]
+            if profile.dense_array is not None else list(),
         }
 
 
@@ -434,6 +435,10 @@ def gather_model_as_jsons(circuit: MultiCircuit) -> Dict[str, Dict[str, str]]:
     :param circuit:
     :return:
     """
+
+    if circuit.has_time_series:
+        circuit.ensure_profiles_exist()
+
     data: Dict[str, Union[Dict[str, str], List[Dict[str, str]]]] = dict()
 
     # declare objects to iterate  name: [sample object, list of objects, headers]
@@ -1040,9 +1045,17 @@ def parse_object_type_from_json(template_elm: ALL_DEV_TYPES,
 
                             if isinstance(gc_prop.tpe, DeviceType):
 
-                                # this is a hyperlink to another object
-                                # we must look for the reference in elements_dict
-                                collection = elements_dict_by_type.get(gc_prop.tpe, None)
+                                if gc_prop.tpe == DeviceType.AnyLineTemplateDevice:
+                                    # this is an exception, when dealing with line templates we need to look for
+                                    # several types, not only one
+                                    seq_templates = elements_dict_by_type.get(DeviceType.OverheadLineTypeDevice, dict())
+                                    oh_templates = elements_dict_by_type.get(DeviceType.SequenceLineDevice, dict())
+                                    ug_templates = elements_dict_by_type.get(DeviceType.UnderGroundLineDevice, dict())
+                                    collection = {**seq_templates, **oh_templates, **ug_templates}
+                                else:
+                                    # this is a hyperlink to another object
+                                    # we must look for the reference in elements_dict
+                                    collection = elements_dict_by_type.get(gc_prop.tpe, None)
 
                                 if collection is not None:
                                     ref_idtag = str(property_value)
@@ -1469,4 +1482,7 @@ def parse_gridcal_data(data: Dict[str, Union[str, float, pd.DataFrame, Dict[str,
     if text_func is not None:
         text_func("Done!")
 
+    if circuit.has_time_series:
+        circuit.ensure_profiles_exist()
+
     return circuit
