@@ -1,5 +1,5 @@
 # GridCal
-# Copyright (C) 2015 - 2023 Santiago Peñate Vera
+# Copyright (C) 2015 - 2024 Santiago Peñate Vera
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -14,17 +14,16 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-from typing import Tuple, Any, Union
+from typing import Any, Union
 
 import chardet
 
 import numpy as np
 import pandas as pd
 
-from GridCalEngine.Core import MultiCircuit
 from GridCalEngine.basic_structures import Logger
-from GridCalEngine.Core.Devices.multi_circuit import MultiCircuit
-import GridCalEngine.Core.Devices as dev
+from GridCalEngine.Devices.multi_circuit import MultiCircuit
+import GridCalEngine.Devices as dev
 
 pd.set_option('display.max_rows', 500)
 pd.set_option('display.max_columns', 500)
@@ -91,43 +90,43 @@ __headers__['CatalogBranch']['ZN'] = ['CLASS', 'EQ', 'DESC', 'VNOM', 'RZN', 'RXN
 
 ########################################################################################################################
 # nodes block
-__headers__['Nodes'] = dict()
+__headers__['Schema'] = dict()
 
 # Airline support post
-__headers__['Nodes']['APOIO'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST']
+__headers__['Schema']['APOIO'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST']
 
 # Cabinet (only for low voltage)
-__headers__['Nodes']['ARM'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'YEAR']
+__headers__['Schema']['ARM'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'YEAR']
 
 # Connection
-__headers__['Nodes']['CX'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST']
+__headers__['Schema']['CX'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST']
 
 # Neutral connection
-__headers__['Nodes']['CXN'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST']
+__headers__['Schema']['CXN'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST']
 
 # Network Equivalent
-__headers__['Nodes']['EQUIV'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'VMIN', 'VMAX', 'ZONE',
+__headers__['Schema']['EQUIV'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'VMIN', 'VMAX', 'ZONE',
                                  'SEPNET', 'AUTOUP', 'P', 'Q', 'ELAST', 'SIMUL', 'HTYP', 'HARM5', 'HARM7', 'HARM11',
                                  'HARM13', 'NOGRW', 'RS', 'XS', 'R1', 'X1', 'R2', 'X2', 'RH', 'XH', 'COM']
 
 # Generator
-__headers__['Nodes']['GEN'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'MODEL', 'VMIN', 'VMAX',
+__headers__['Schema']['GEN'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'MODEL', 'VMIN', 'VMAX',
                                'V', 'ENAB', 'P', 'Q', 'QMIN', 'QMAX', 'ELAST', 'HTYP', 'HARM5', 'HARM7', 'HARM11',
                                'HARM13', 'VNOM', 'RAT', 'TGEN', 'COST', 'YEAR']
 
 # Charging (only for low voltage)
-__headers__['Nodes']['LOAD'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'VMIN', 'VMAX',
+__headers__['Schema']['LOAD'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'VMIN', 'VMAX',
                                 'NCMPLAN']  # fill to fit...
 
 # Transformation station
-__headers__['Nodes']['PT'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'VMIN', 'VMAX', 'ZONE',
+__headers__['Schema']['PT'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'VMIN', 'VMAX', 'ZONE',
                               'ENAB', 'P', 'Q', 'ELAST', 'SIMUL', 'HTYP', 'HARM5', 'HARM7', 'HARM11', 'HARM13', 'NOGRW',
                               'EQEXIST', 'EQPOSS1', 'MCOST1', 'ICOST1', 'EQPOSS2', 'MCOST2', 'ICOST2', 'EQPOSS3',
                               'MCOST3',
                               'ICOST3', 'NCLI', 'EQTYPE', 'YEAR', 'COM', 'INFOCOM', 'ID_AUX']
 
 # Customer transformation office
-__headers__['Nodes']['PTC'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'VMIN', 'VMAX', 'ZONE',
+__headers__['Schema']['PTC'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'VMIN', 'VMAX', 'ZONE',
                                'ENAB', 'P', 'Q', 'ELAST', 'SIMUL', 'HTYP', 'HARM5', 'HARM7', 'HARM11', 'HARM13',
                                'NOGRW',
                                'EQEXIST', 'EQPOSS1', 'MCOST1', 'ICOST1', 'EQPOSS2', 'MCOST2', 'ICOST2', 'EQPOSS3',
@@ -135,11 +134,11 @@ __headers__['Nodes']['PTC'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX',
                                'ICOST3', 'NCLI', 'EQTYPE', 'YEAR', 'COM', 'INFOCOM', 'ID_AUX']
 
 # Reference node
-__headers__['Nodes']['REF'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'VREF', 'RAT',
+__headers__['Schema']['REF'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'VREF', 'RAT',
                                'COST', 'TGEN', 'YEAR']
 
 # Voltage Transformer
-__headers__['Nodes']['TT'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'VMIN', 'VMAX',
+__headers__['Schema']['TT'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'VMIN', 'VMAX',
                               'DISABLE', 'HARM5', 'HARM7', 'HARM11', 'HARM13', 'EQEXIST', 'TAP', 'YEAR', 'ID_AUX']
 
 ########################################################################################################################
@@ -252,7 +251,7 @@ def read_dpx_data(file_name):
 
                 if len(values) > 1:
 
-                    if current_block in ['CatalogNode', 'CatalogBranch', 'Areas', 'Sites', 'Nodes',
+                    if current_block in ['CatalogNode', 'CatalogBranch', 'Areas', 'Sites', 'Schema',
                                          'Branches']:  # blocks with further categorization
 
                         # check the if the block has been created
@@ -313,7 +312,7 @@ def repack(data_structures, logger=Logger(), verbose=False):
             # blocks without header marker
             pass
 
-        elif current_block in ['Nodes', 'Branches', 'CatalogBranch']:  # blocks without header marker
+        elif current_block in ['Schema', 'Branches', 'CatalogBranch']:  # blocks without header marker
 
             # repack the data with headers
             for tpe in data_structures[current_block].keys():
@@ -390,15 +389,15 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
 
     buses_id_dict = dict()
     #  create nodes
-    for tpe in data_structures['Nodes']:
+    for tpe in data_structures['Schema']:
         # Airline support post
-        # __headers__['Nodes']['APOIO'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST']
-        # __headers__['Nodes']['ARM'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'YEAR']
-        # __headers__['Nodes']['CX'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST']
-        # __headers__['Nodes']['CXN'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST']
-        # __headers__['Nodes']['LOAD'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'VMIN', 'VMAX', 'NCMPLAN']  # fill to fit...
+        # __headers__['Schema']['APOIO'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST']
+        # __headers__['Schema']['ARM'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'YEAR']
+        # __headers__['Schema']['CX'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST']
+        # __headers__['Schema']['CXN'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST']
+        # __headers__['Schema']['LOAD'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'VMIN', 'VMAX', 'NCMPLAN']  # fill to fit...
         if tpe in ['APOIO', 'ARM', 'CX', 'CXN', 'LOAD']:
-            df = data_structures['Nodes'][tpe]
+            df = data_structures['Schema'][tpe]
 
             for i in range(df.shape[0]):
                 name = 'B' + str(len(circuit.buses) + 1) + '_' + str(df['NAME'].values[i])
@@ -406,18 +405,18 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
                 x = float(df['GX'].values[i]) / contraction_factor
                 y = float(df['GY'].values[i]) / contraction_factor
                 id_ = df['ID'].values[i]
-                bus = dev.Bus(name=name, vnom=Vnom, xpos=x, ypos=y, height=40, width=60)
+                bus = dev.Bus(name=name, Vnom=Vnom, xpos=x, ypos=y, height=40, width=60)
 
                 circuit.add_bus(bus)
                 buses_id_dict[id_] = bus
 
         # Network Equivalent
-        # __headers__['Nodes']['EQUIV'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'VMIN', 'VMAX', 'ZONE',
+        # __headers__['Schema']['EQUIV'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'VMIN', 'VMAX', 'ZONE',
         #                                  'SEPNET', 'AUTOUP', 'P', 'Q', 'ELAST', 'SIMUL', 'HTYP', 'HARM5', 'HARM7',
         #                                  'HARM11',
         #                                  'HARM13', 'NOGRW', 'RS', 'XS', 'R1', 'X1', 'R2', 'X2', 'RH', 'XH', 'COM']
         elif tpe == 'EQUIV':
-            df = data_structures['Nodes'][tpe]
+            df = data_structures['Schema'][tpe]
 
             for i in range(df.shape[0]):
                 name = 'B' + str(len(circuit.buses) + 1) + '_' + str(df['NAME'].values[i])
@@ -425,7 +424,7 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
                 x = float(df['GX'].values[i]) / contraction_factor
                 y = float(df['GY'].values[i]) / contraction_factor
                 id_ = df['ID'].values[i]
-                bus = dev.Bus(name=name, vnom=Vnom, xpos=x, ypos=y, height=40, width=60, is_slack=True)
+                bus = dev.Bus(name=name, Vnom=Vnom, xpos=x, ypos=y, height=40, width=60, is_slack=True)
                 circuit.add_bus(bus)
                 buses_id_dict[id_] = bus
 
@@ -437,13 +436,13 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
                 circuit.add_load(bus, load)
 
         # Generator
-        # __headers__['Nodes']['GEN'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'MODEL', 'VMIN',
+        # __headers__['Schema']['GEN'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'MODEL', 'VMIN',
         #                                'VMAX',
         #                                'V', 'ENAB', 'P', 'Q', 'QMIN', 'QMAX', 'ELAST', 'HTYP', 'HARM5', 'HARM7',
         #                                'HARM11',
         #                                'HARM13', 'VNOM', 'RAT', 'TGEN', 'COST', 'YEAR']
         elif tpe == 'GEN':
-            df = data_structures['Nodes'][tpe]
+            df = data_structures['Schema'][tpe]
 
             for i in range(df.shape[0]):
                 name = 'B' + str(len(circuit.buses) + 1) + '_' + str(df['NAME'].values[i])
@@ -451,7 +450,7 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
                 x = float(df['GX'].values[i]) / contraction_factor
                 y = float(df['GY'].values[i]) / contraction_factor
                 id_ = df['ID'].values[i]
-                bus = dev.Bus(name=name, vnom=Vnom, xpos=x, ypos=y, height=40, width=60)
+                bus = dev.Bus(name=name, Vnom=Vnom, xpos=x, ypos=y, height=40, width=60)
                 circuit.add_bus(bus)
                 buses_id_dict[id_] = bus
 
@@ -473,7 +472,7 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
                     circuit.add_static_generator(bus, gen)
 
         # Transformation station
-        # __headers__['Nodes']['PT'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'VMIN', 'VMAX',
+        # __headers__['Schema']['PT'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'VMIN', 'VMAX',
         #                               'ZONE',
         #                               'ENAB', 'P', 'Q', 'ELAST', 'SIMUL', 'HTYP', 'HARM5', 'HARM7', 'HARM11', 'HARM13',
         #                               'NOGRW',
@@ -482,7 +481,7 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
         #                               'ICOST3', 'NCLI', 'EQTYPE', 'YEAR', 'COM', 'INFOCOM', 'ID_AUX']
         elif tpe in ['PT', 'PTC']:
 
-            df = data_structures['Nodes'][tpe]
+            df = data_structures['Schema'][tpe]
 
             for i in range(df.shape[0]):
                 name = 'B' + str(len(circuit.buses) + 1) + '_' + str(df['NAME'].values[i])
@@ -490,7 +489,7 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
                 x = float(df['GX'].values[i]) / contraction_factor
                 y = float(df['GY'].values[i]) / contraction_factor
                 id_ = df['ID'].values[i]
-                bus = dev.Bus(name=name, vnom=Vnom, xpos=x, ypos=y, height=40, width=60)
+                bus = dev.Bus(name=name, Vnom=Vnom, xpos=x, ypos=y, height=40, width=60)
 
                 name = 'LD' + str(len(circuit.buses) + 1) + '_' + str(df['NAME'].values[i])
                 p = float(df['P'].values[i]) * Sbase
@@ -502,10 +501,10 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
                 buses_id_dict[id_] = bus
 
         # Reference node
-        # __headers__['Nodes']['REF'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'VREF', 'RAT',
+        # __headers__['Schema']['REF'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'VREF', 'RAT',
         #                                'COST', 'TGEN', 'YEAR']
         elif tpe == 'REF':
-            df = data_structures['Nodes'][tpe]
+            df = data_structures['Schema'][tpe]
 
             for i in range(df.shape[0]):
                 name = 'B' + str(len(circuit.buses) + 1) + '_' + str(df['NAME'].values[i])
@@ -513,17 +512,17 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
                 x = float(df['GX'].values[i]) / contraction_factor
                 y = float(df['GY'].values[i]) / contraction_factor
                 id_ = df['ID'].values[i]
-                bus = dev.Bus(name=name, vnom=Vnom, xpos=x, ypos=y, height=40, width=60, is_slack=True)
+                bus = dev.Bus(name=name, Vnom=Vnom, xpos=x, ypos=y, height=40, width=60, is_slack=True)
 
                 circuit.add_bus(bus)
                 buses_id_dict[id_] = bus
 
         # Voltage Transformer
-        # __headers__['Nodes']['TT'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'VMIN', 'VMAX',
+        # __headers__['Schema']['TT'] = ['CLASS', 'ID', 'NAME', 'VBASE', 'GX', 'GY', 'SX', 'SY', 'EXIST', 'VMIN', 'VMAX',
         #                               'DISABLE', 'HARM5', 'HARM7', 'HARM11', 'HARM13', 'EQEXIST', 'TAP', 'YEAR',
         #                               'ID_AUX']
         elif tpe == 'TT':
-            df = data_structures['Nodes'][tpe]
+            df = data_structures['Schema'][tpe]
 
             for i in range(df.shape[0]):
                 name = 'B' + str(len(circuit.buses) + 1) + '_' + str(df['NAME'].values[i])
@@ -531,13 +530,13 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
                 x = float(df['GX'].values[i]) / contraction_factor
                 y = float(df['GY'].values[i]) / contraction_factor
                 id_ = df['ID'].values[i]
-                bus = dev.Bus(name=name, vnom=Vnom, xpos=x, ypos=y, height=40, width=60)
+                bus = dev.Bus(name=name, Vnom=Vnom, xpos=x, ypos=y, height=40, width=60)
 
                 circuit.add_bus(bus)
                 buses_id_dict[id_] = bus
 
         else:
-            logger.add_error('Not recognised under Nodes', tpe)
+            logger.add_error('Not recognised under Schema', tpe)
 
     # create Branches
     for tpe in data_structures['Branches']:
@@ -562,11 +561,11 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
 
                 try:
                     x = float(cat_elm['REAC'].values[0]) * Sbase
-                except:
+                except ValueError as e:
                     x = 1e-20
 
-                br = dev.Branch(bus_from=b1, bus_to=b2, name=name, x=x, branch_type=dev.BranchType.Branch)
-                circuit.add_branch(br)
+                br = dev.SeriesReactance(bus_from=b1, bus_to=b2, name=name, x=x)
+                circuit.add_series_reactance(br)
 
         # Estimator
         # __headers__['Branches']['ESTIM'] = ['CLASS', 'ID', 'NAME', 'ID1', 'ID2', 'INDEP', 'I', 'SIMULT']
@@ -607,8 +606,8 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
                 state = bool(int(df['STAT'].values[i]))
                 b1 = buses_id_dict[id1]
                 b2 = buses_id_dict[id2]
-                br = dev.Branch(bus_from=b1, bus_to=b2, name=name, active=state, branch_type=dev.BranchType.Switch)
-                circuit.add_branch(br)
+                br = dev.Switch(bus_from=b1, bus_to=b2, name=name, active=state)
+                circuit.add_switch(br)
 
         # Lines, cables and bars
         # fill until it fits or truncate the data
@@ -637,15 +636,15 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
 
                 try:
                     r = float(cat_elm['R'].values[0]) * length / 1000
-                except:
+                except ValueError as e:
                     r = 1e-20
                 try:
                     x = float(cat_elm['X'].values[0]) * length / 1000
-                except:
+                except ValueError as e:
                     x = 1e-20
                 try:
                     b = float(cat_elm['B'].values[0]) * length / 1000
-                except:
+                except ValueError as e:
                     b = 1e-20
 
                 Imax = float(cat_elm['RATTYP'].values[0]) / 1000.0  # pass from A to kA
@@ -656,9 +655,8 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
                 x = x if x > 0.0 else 1e-20
                 b = b if b > 0.0 else 1e-20
 
-                br = dev.Branch(bus_from=b1, bus_to=b2, name=name, r=r, x=x, b=b, rate=Smax, length=length,
-                                branch_type=dev.BranchType.Line)
-                circuit.add_branch(br)
+                br = dev.Line(bus_from=b1, bus_to=b2, name=name, r=r, x=x, b=b, rate=Smax, length=length)
+                circuit.add_line(br)
 
         # Intensity Transformer
         # __headers__['Branches']['TI'] = ['CLASS', 'ID', 'NAME', 'ID1', 'ID2', 'INDEP', 'I', 'SIMULT', 'EXIST', 'STAT',
@@ -678,8 +676,8 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
                 df_cat = data_structures['CatalogBranch'][tpe]
                 cat_elm = df_cat[df_cat['EQ'] == eq_id]
 
-                br = dev.Branch(bus_from=b1, bus_to=b2, name=name, branch_type=dev.BranchType.Transformer)
-                circuit.add_branch(br)
+                br = dev.Transformer2W(bus_from=b1, bus_to=b2, name=name)
+                circuit.add_transformer2w(br)
 
         # Self-transformer
         # __headers__['Branches']['XFORM1'] = ['CLASS', 'ID', 'NAME', 'ID1', 'ID2', 'ID3', 'ID1N', 'ID2N', 'ID3N',
@@ -727,9 +725,8 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
                     s = 1e-20
                     logger.add_error('Not found.', tpe + ':' + eq_id)
 
-                br = dev.Branch(bus_from=b1, bus_to=b2, name=name, r=r, x=x, rate=s,
-                                branch_type=dev.BranchType.Transformer)
-                circuit.add_branch(br)
+                br = dev.Transformer2W(bus_from=b1, bus_to=b2, name=name, r=r, x=x, rate=s)
+                circuit.add_transformer2w(br)
 
         # 3-winding transformer
         # __headers__['Branches']['XFORM3'] = ['CLASS', 'ID', 'NAME', 'ID1', 'ID2', 'ID3', 'ID1N', 'ID2N', 'ID3N',
@@ -791,17 +788,11 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
                 x23 = x23 if x23 > 0.0 else 1e-20
                 s23 = s23 if s23 > 0.0 else 1e-20
 
-                br = dev.Branch(bus_from=b1, bus_to=b2, name=name, r=r12, x=x12, rate=s12,
-                                branch_type=dev.BranchType.Transformer)
-                circuit.add_branch(br)
-
-                br = dev.Branch(bus_from=b1, bus_to=b3, name=name, r=r13, x=x13, rate=s13,
-                                branch_type=dev.BranchType.Transformer)
-                circuit.add_branch(br)
-
-                br = dev.Branch(bus_from=b2, bus_to=b3, name=name, r=r23, x=x23, rate=s23,
-                                branch_type=dev.BranchType.Transformer)
-                circuit.add_branch(br)
+                circuit.add_transformer3w(obj=dev.Transformer3W(bus1=b1, bus2=b2, bus3=b3, name=name,
+                                                                r12=r12, x12=x12,
+                                                                r23=r23, x23=x23,
+                                                                r31=r13, x31=x13,
+                                                                rate12=s1, rate23=s2, rate31=s3))
 
         # Neutral impedance
         # __headers__['Branches']['ZN'] = ['CLASS', 'ID', 'NAME', 'ID1', 'ID2', 'EXIST', 'STAT', 'PERM', 'FAILRT',
@@ -815,8 +806,8 @@ def load_dpx(file_name, contraction_factor=1000) -> tuple[Union[MultiCircuit, An
                 id2 = df['ID2'].values[i]
                 b1 = buses_id_dict[id1]
                 b2 = buses_id_dict[id2]
-                br = dev.Branch(bus_from=b1, bus_to=b2, name=name, branch_type=dev.BranchType.Branch)
-                circuit.add_branch(br)
+                br = dev.SeriesReactance(bus_from=b1, bus_to=b2, name=name)
+                circuit.add_series_reactance(br)
 
     # return the circuit and the logs
     return circuit, logger
