@@ -39,6 +39,7 @@ from GridCalEngine.IO.raw.devices.gne_device import RawGneDevice
 from GridCalEngine.IO.raw.devices.system_switching_device import RawSystemSwitchingDevice
 from GridCalEngine.IO.base.base_circuit import BaseCircuit
 from GridCalEngine.IO.raw.devices.psse_circuit import PsseCircuit
+from GridCalEngine.enumerations import TapChangerTypes
 
 
 def get_gridcal_bus(psse_bus: RawBus,
@@ -369,6 +370,16 @@ def get_gridcal_transformer(psse_elm: RawTransformer,
                                 mttf=0,
                                 mttr=0)
 
+        if psse_elm.COD1 == 0:
+            elm.tap_changer.total_positions = 2
+            elm.tap_changer.neutral_position = 0
+            elm.tap_changer.tap_position = -1
+            elm.tap_changer.dV = round(1 - tap_module, 6)
+            elm.tap_changer.asymmetry_angle = 90.0
+            elm.tap_changer.tc_type = TapChangerTypes.NoRegulation
+        else:
+            pass
+
         return elm, 2
 
     elif psse_elm.windings == 3:
@@ -531,7 +542,7 @@ def get_hvdc_from_vscdc(psse_elm: RawVscDCLine,
         bus2 = psse_bus_dict[IBUS2]
 
         name1 = psse_elm.NAME.replace("'", "").replace('/', '').strip()
-        idtag = str(psse_elm.IBUS1) + '_' + str(psse_elm.IBUS2) + '_1'
+        code = str(psse_elm.IBUS1) + '_' + str(psse_elm.IBUS2) + '_1'
 
         Vset_f = psse_elm.ACSET1
         Vset_t = psse_elm.ACSET2
@@ -548,7 +559,7 @@ def get_hvdc_from_vscdc(psse_elm: RawVscDCLine,
         obj = dev.HvdcLine(bus_from=bus1,
                            bus_to=bus2,
                            name=name1,
-                           idtag=idtag,
+                           code=code,
                            Pset=specified_power,
                            Vset_f=Vset_f,
                            Vset_t=Vset_t,
@@ -598,7 +609,7 @@ def get_hvdc_from_twotermdc(psse_elm: RawTwoTerminalDCLine,
         Vset_t = 1.0
 
         name1 = psse_elm.NAME.replace("'", "").replace('"', "").replace('/', '').strip()
-        idtag = str(psse_elm.IPR) + '_' + str(psse_elm.IPI) + '_1'
+        code = str(psse_elm.IPR) + '_' + str(psse_elm.IPI) + '_1'
 
         # set the HVDC line active
         active = bus1.active and bus2.active
@@ -607,7 +618,7 @@ def get_hvdc_from_twotermdc(psse_elm: RawTwoTerminalDCLine,
                            bus_to=bus2,  # inverter as of PSSe
                            active=active,
                            name=name1,
-                           idtag=idtag,
+                           code=code,
                            Pset=specified_power,
                            Vset_f=Vset_f,
                            Vset_t=Vset_t,
