@@ -14,6 +14,8 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+from __future__ import annotations
+
 import numpy as np
 from typing import Union, List
 
@@ -201,8 +203,8 @@ class AvailableTransferCapacityTimeSeriesDriver(TimeSeriesDriverTemplate):
 
     def __init__(self,
                  grid: MultiCircuit,
-                 options: AvailableTransferCapacityOptions,
-                 time_indices: np.ndarray,
+                 options: AvailableTransferCapacityOptions | None,
+                 time_indices: DateVec,
                  clustering_results: Union[ClusteringResults, None] = None):
 
         """
@@ -308,8 +310,8 @@ class AvailableTransferCapacityTimeSeriesDriver(TimeSeriesDriverTemplate):
                                   Pinstalled=nc.bus_installed_power,
                                   Pgen=nc.generator_data.get_injections_per_bus().real,
                                   Pload=nc.load_data.get_injections_per_bus().real,
-                                  idx1=self.options.bus_idx_from,
-                                  idx2=self.options.bus_idx_to,
+                                  bus_a1_idx=self.options.bus_idx_from,
+                                  bus_a2_idx=self.options.bus_idx_to,
                                   mode=mode_2_int[self.options.mode])
 
             # base exchange
