@@ -15,7 +15,7 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-import numpy as np
+from typing import List
 from GridCalEngine.Devices.multi_circuit import MultiCircuit
 from GridCalEngine.Simulations.NTC.ntc_opf import run_linear_ntc_opf_ts
 from GridCalEngine.Simulations.driver_template import DriverTemplate
@@ -46,13 +46,13 @@ class OptimalNetTransferCapacityDriver(DriverTemplate):
 
         self.logger = Logger()
 
-    def get_steps(self):
+    def get_steps(self) -> List[int]:
         """
         Get time steps list of strings
         """
         return list()
 
-    def opf(self):
+    def opf(self) -> OptimalNetTransferCapacityResults:
         """
         Run a power flow for every circuit
         @return: OptimalPowerFlowResults object
@@ -69,9 +69,10 @@ class OptimalNetTransferCapacityDriver(DriverTemplate):
             zonal_grouping=self.options.opf_options.zonal_grouping,
             skip_generation_limits=self.options.skip_generation_limits,
             consider_contingencies=self.options.consider_contingencies,
+            contingency_groups_used=self.options.opf_options.contingency_groups_used,
             lodf_threshold=self.options.lin_options.lodf_threshold,
-            buses_areas_1=self.options.area_from_bus_idx,
-            buses_areas_2=self.options.area_to_bus_idx,
+            bus_a1_idx=self.options.sending_bus_idx,
+            bus_a2_idx=self.options.receiving_bus_idx,
             transfer_method=self.options.transfer_method,
             monitor_only_ntc_load_rule_branches=self.options.use_branch_rating_contribution,
             alpha_threshold=self.options.branch_exchange_sensitivity,
@@ -80,93 +81,53 @@ class OptimalNetTransferCapacityDriver(DriverTemplate):
             logger=self.logger,
             progress_text=self.report_text,
             progress_func=self.report_progress,
-            export_model_fname=self.options.opf_options.export_model_fname
+            export_model_fname=self.options.opf_options.export_model_fname,
+            verbose=self.options.opf_options.verbose,
+            robust=self.options.opf_options.robust
         )
 
-        inter_area_branches = self.grid.get_inter_areas_branches(a1=self.options.area_from_bus_idx,
-                                                                 a2=self.options.area_to_bus_idx)
-
-        inter_area_hvdc = self.grid.get_inter_areas_hvdc_branches(a1=self.options.area_from_bus_idx,
-                                                                  a2=self.options.area_to_bus_idx)
-
         # pack the results
         self.results = OptimalNetTransferCapacityResults(
             bus_names=self.grid.get_bus_names(),
             branch_names=self.grid.get_branch_names_wo_hvdc(),
-            load_names=self.grid.get_load_names(),
-            generator_names=self.grid.get_generator_names(),
-            battery_names=self.grid.get_battery_names(),
             hvdc_names=self.grid.get_hvdc_names(),
-            trm=self.options.transmission_reliability_margin,
-            ntc_load_rule=self.options.branch_rating_contribution,
-            Sbus=opf_vars.bus_vars.Pcalc[0, :],
-            voltage=opf_vars.get_voltages()[0, :],
-            Sf=opf_vars.branch_vars.flows[0, :],
-            loading=opf_vars.branch_vars.loading[0, :],
-            solved=bool(opf_vars.acceptable_solution),
-            bus_types=np.ones(self.grid.get_bus_number(), dtype=int),
-            hvdc_flow=opf_vars.hvdc_vars.flows[0, :],
-            hvdc_loading=opf_vars.hvdc_vars.loading[0, :],
-            phase_shift=opf_vars.branch_vars.tap_angles[0, :],
-            inter_area_branches=inter_area_branches,
-            inter_area_hvdc=inter_area_hvdc,
-            # alpha=alpha,
-            # alpha_n1=alpha_n1,
-            monitor=opf_vars.branch_vars.monitor,
-            monitor_type=opf_vars.branch_vars.monitor_type,
-            # contingency_branch_flows_list=problem.get_contingency_flows_list(),
-            # contingency_branch_indices_list=problem.contingency_indices_list,
-            # contingency_branch_alpha_list=problem.contingency_branch_alpha_list,
-            # contingency_generation_flows_list=problem.get_contingency_gen_flows_list(),
-            # contingency_generation_indices_list=problem.contingency_gen_indices_list,
-            # contingency_generation_alpha_list=problem.contingency_generation_alpha_list,
-            # contingency_hvdc_flows_list=problem.get_contingency_hvdc_flows_list(),
-            # contingency_hvdc_indices_list=problem.contingency_hvdc_indices_list,
-            # contingency_hvdc_alpha_list=problem.contingency_hvdc_alpha_list,
-            # branch_ntc_load_rule=problem.get_branch_ntc_load_rule(),
-            rates=opf_vars.branch_vars.rates,
-            contingency_rates=opf_vars.branch_vars.rates,  # TODO set contingency rates
-            area_from_bus_idx=self.options.area_from_bus_idx,
-            area_to_bus_idx=self.options.area_to_bus_idx,
-            # structural_ntc=problem.structural_ntc,
-            sbase=self.grid.Sbase,
-            loading_threshold=self.options.loading_threshold_to_report,
-            reversed_sort_loading=False,
-            branch_control_modes=[],
-            hvdc_control_modes=[],
+            contingency_group_names=self.grid.get_contingency_group_names()
         )
 
-        self.results.voltage = np.ones((opf_vars.nt, opf_vars.nbus)) * np.exp(1j * opf_vars.bus_vars.theta[0, :])
+        self.results.voltage = opf_vars.get_voltages()[0, :]
+        self.results.Sbus = opf_vars.bus_vars.Pcalc[0, :]
+        self.results.dSbus = opf_vars.bus_vars.delta_p[0, :]
         self.results.bus_shadow_prices = opf_vars.bus_vars.shadow_prices[0, :]
-        # self.results.load_shedding = opf_vars.load_vars.shedding
-        # self.results.battery_power = opf_vars.batt_vars.p[0, :]
-        # self.results.battery_energy = opf_vars.batt_vars.e[0, :]
-        # self.results.generator_power = opf_vars.gen_vars.p[0, :]
+        self.results.load_shedding = opf_vars.bus_vars.load_shedding[0, :]
+
         self.results.Sf = opf_vars.branch_vars.flows[0, :]
         self.results.St = -opf_vars.branch_vars.flows[0, :]
         self.results.overloads = opf_vars.branch_vars.flow_slacks_pos[0, :] - opf_vars.branch_vars.flow_slacks_neg[0, :]
         self.results.loading = opf_vars.branch_vars.loading[0, :]
         self.results.phase_shift = opf_vars.branch_vars.tap_angles[0, :]
-        # self.results.Sbus = problem.get_power_injections()
+        self.results.rates = opf_vars.branch_vars.rates[0, :]
+        self.results.contingency_rates = opf_vars.branch_vars.contingency_rates[0, :]
+        self.results.alpha = opf_vars.branch_vars.alpha[0, :]
+        self.results.monitor_logic = opf_vars.branch_vars.monitor_logic[0, :]
+        self.results.contingency_flows_list = opf_vars.branch_vars.contingency_flow_data
+
         self.results.hvdc_Pf = opf_vars.hvdc_vars.flows[0, :]
         self.results.hvdc_loading = opf_vars.hvdc_vars.loading[0, :]
 
-        self.report_text('Creating reports...')
+        self.results.sending_bus_idx = self.options.sending_bus_idx
+        self.results.receiving_bus_idx = self.options.receiving_bus_idx
+        self.results.inter_space_branches = opf_vars.branch_vars.inter_space_branches
+        self.results.inter_space_hvdc = opf_vars.hvdc_vars.inter_space_hvdc
 
-        # TODO Fix this
-        # self.results.create_all_reports(
-        #     loading_threshold=self.options.loading_threshold_to_report,
-        #     reverse=self.options.reversed_sort_loading,
-        # )
+        self.results.converged = opf_vars.acceptable_solution
 
         self.report_text('Done!')
 
         return self.results
 
-    def run(self):
+    def run(self) -> None:
         """
-
-        :return:
+        Run this study
         """
         self.tic()
 
