@@ -14,6 +14,7 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+from __future__ import annotations
 
 from GridCalEngine.Simulations.options_template import OptionsTemplate
 from GridCalEngine.Simulations.OPF.opf_options import OptimalPowerFlowOptions
@@ -28,23 +29,23 @@ class OptimalNetTransferCapacityOptions(OptionsTemplate):
     """
 
     def __init__(self,
-                 area_from_bus_idx: IntVec,
-                 area_to_bus_idx: IntVec,
-                 transfer_method: AvailableTransferMode,
-                 loading_threshold_to_report: float,
-                 skip_generation_limits: bool,
-                 transmission_reliability_margin: float,
-                 branch_exchange_sensitivity: float,
-                 use_branch_exchange_sensitivity: bool,
-                 branch_rating_contribution: float,
-                 use_branch_rating_contribution: bool,
-                 consider_contingencies: bool,
-                 opf_options: OptimalPowerFlowOptions,
-                 lin_options: LinearAnalysisOptions):
+                 sending_bus_idx: IntVec,
+                 receiving_bus_idx: IntVec,
+                 transfer_method: AvailableTransferMode = AvailableTransferMode.InstalledPower,
+                 loading_threshold_to_report: float = 98.0,
+                 skip_generation_limits: bool = True,
+                 transmission_reliability_margin: float = 400.0,
+                 branch_exchange_sensitivity: float = 5.0 / 100.0,
+                 use_branch_exchange_sensitivity: bool = True,
+                 branch_rating_contribution: float = 70 / 100.0,
+                 use_branch_rating_contribution: bool = False,
+                 consider_contingencies: bool = False,
+                 opf_options: OptimalPowerFlowOptions | None = None,
+                 lin_options: LinearAnalysisOptions | None = None, ):
         """
 
-        :param area_from_bus_idx: array of area "from" bus indices
-        :param area_to_bus_idx: array of area "to" bus indices
+        :param sending_bus_idx: array of area "from" bus indices
+        :param receiving_bus_idx: array of area "to" bus indices
         :param transfer_method: AvailableTransferMode
         :param loading_threshold_to_report:
         :param skip_generation_limits:
@@ -59,8 +60,8 @@ class OptimalNetTransferCapacityOptions(OptionsTemplate):
         """
         OptionsTemplate.__init__(self, name="OptimalNetTransferCapacityOptions")
 
-        self.area_from_bus_idx: IntVec = area_from_bus_idx
-        self.area_to_bus_idx: IntVec = area_to_bus_idx
+        self.sending_bus_idx: IntVec = sending_bus_idx
+        self.receiving_bus_idx: IntVec = receiving_bus_idx
 
         self.transfer_method: AvailableTransferMode = transfer_method
         self.loading_threshold_to_report: float = loading_threshold_to_report
@@ -72,11 +73,18 @@ class OptimalNetTransferCapacityOptions(OptionsTemplate):
         self.use_branch_rating_contribution: bool = use_branch_rating_contribution
         self.consider_contingencies: bool = consider_contingencies
 
-        self.opf_options: OptimalPowerFlowOptions = opf_options
-        self.lin_options: LinearAnalysisOptions = lin_options
+        if opf_options is None:
+            self.opf_options = OptimalPowerFlowOptions()
+        else:
+            self.opf_options: OptimalPowerFlowOptions = opf_options
 
-        self.register(key="area_from_bus_idx", tpe=SubObjectType.Array)
-        self.register(key="area_to_bus_idx", tpe=SubObjectType.Array)
+        if lin_options is None:
+            self.lin_options = LinearAnalysisOptions()
+        else:
+            self.lin_options: LinearAnalysisOptions = lin_options
+
+        self.register(key="sending_bus_idx", tpe=SubObjectType.Array)
+        self.register(key="receiving_bus_idx", tpe=SubObjectType.Array)
         self.register(key="transfer_method", tpe=AvailableTransferMode)
         self.register(key="loading_threshold_to_report", tpe=float)
         self.register(key="skip_generation_limits", tpe=bool)
