@@ -15,19 +15,17 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-"""
-Solves the power flow using a Gauss-Seidel method.
-"""
-
 import time
 import numpy as np
 import GridCalEngine.Simulations.PowerFlow.NumericalMethods.common_functions as cf
 from GridCalEngine.Simulations.PowerFlow.power_flow_results import NumericPowerFlowResults
+from GridCalEngine.Simulations.PowerFlow.NumericalMethods.discrete_controls import (control_q_inside_method,
+                                                                                    compute_slack_distribution)
 from GridCalEngine.basic_structures import Logger
 
 
-def gausspf(Ybus, S0, I0, Y0, V0, pv, pq, tol=1e-3, max_it=50,
-            verbose=False, logger: Logger = None) -> NumericPowerFlowResults:
+def gausspf(Ybus, S0, I0, Y0, V0, pv, pq, p, pqv, vd, bus_installed_power, Qmin, Qmax, tol=1e-3, max_it=50,
+            control_q=False, distribute_slack=False, verbose=False, logger: Logger = None) -> NumericPowerFlowResults:
     """
     Gauss-Seidel Power flow
     :param Ybus: Admittance matrix
@@ -37,8 +35,16 @@ def gausspf(Ybus, S0, I0, Y0, V0, pv, pq, tol=1e-3, max_it=50,
     :param V0: Voltage seed solution array
     :param pv: array of pv-node indices
     :param pq: array of pq-node indices
+    :param p: array of p-node indices
+    :param pqv: array of pqv-node indices
+    :param vd: array of vd-node indices
+    :param bus_installed_power: array of bus installed power
+    :param Qmin: Minimum Q limits per bus
+    :param Qmax: Maximum Q limits per bus
     :param tol: Tolerance
     :param max_it: Maximum number of iterations
+    :param control_q: Control Q limits?
+    :param distribute_slack: Distribute Slack?
     :param verbose: Verbose?
     :param logger: Logger to store the debug information
     :return: NumericPowerFlowResults instance
@@ -54,7 +60,6 @@ def gausspf(Ybus, S0, I0, Y0, V0, pv, pq, tol=1e-3, max_it=50,
 
     # set up indexing for updating V
     npv = len(pv)
-    npq = len(pq)
     pvpq = np.r_[pv, pq]
 
     # evaluate F(x0)
@@ -96,6 +101,35 @@ def gausspf(Ybus, S0, I0, Y0, V0, pv, pq, tol=1e-3, max_it=50,
         # check for convergence
         converged = normF < tol
 
+        # control of Q limits --------------------------------------------------------------------------------------
+        # review reactive power limits
+        # it is only worth checking Q limits with a low error
+        # since with higher errors, the Q values may be far from realistic
+        # finally, the Q control only makes sense if there are pv nodes
+        if control_q and normF < 1e-2 and (len(pv) + len(p)) > 0:
+
+            # check and adjust the reactive power
+            # this function passes pv buses to pq when the limits are violated,
+            # but not pq to pv because that is unstable
+            changed, pv, pq, pqv, p = control_q_inside_method(Scalc, S0, pv, pq, pqv, p, Qmin, Qmax)
+
+            if len(changed) > 0:
+                # adjust internal variables to the new pq|pv values
+                F = cf.compute_fx(Scalc, Sbus, pvpq, pq)
+                normF = cf.compute_fx_error(F)
+                converged = normF < tol
+
+        if distribute_slack and normF < 1e-2:
+            ok, delta = compute_slack_distribution(Scalc=Scalc,
+                                                   vd=vd,
+                                                   bus_installed_power=bus_installed_power)
+            if ok:
+                S0 += delta
+                Sbus = cf.compute_zip_power(S0, I0, Y0, Vm)
+                F = cf.compute_fx(Scalc, Sbus, pvpq, pq)
+                normF = cf.compute_fx_error(F)
+                converged = normF < tol
+
         if verbose:
             logger.add_debug('GS Iteration {0}'.format(iter_) + '-' * 200)
 
@@ -111,8 +145,7 @@ def gausspf(Ybus, S0, I0, Y0, V0, pv, pq, tol=1e-3, max_it=50,
     end = time.time()
     elapsed = end - start
 
-    # return NumericPowerFlowResults(V, converged, normF, Scalc, None, None, None, None, None, None, iter_, elapsed)
     return NumericPowerFlowResults(V=V, converged=converged, norm_f=normF,
-                                   Scalc=Scalc, ma=None, theta=None, Beq=None,
+                                   Scalc=Scalc, m=None, tau=None, Beq=None,
                                    Ybus=None, Yf=None, Yt=None,
                                    iterations=iter_, elapsed=elapsed)
