@@ -36,10 +36,10 @@ from GridCalEngine.DataStructures.numerical_circuit import compile_numerical_cir
 from GridCalEngine.Simulations.PowerFlow.NumericalMethods.discrete_controls import compute_slack_distribution
 from GridCalEngine.Devices.Substation.bus import Bus
 from GridCalEngine.Devices.Aggregation.area import Area
-from GridCalEngine.basic_structures import CxVec, CscMat
+from GridCalEngine.basic_structures import CxVec, CscMat, Vec
 
 if TYPE_CHECKING:  # Only imports the below statements during type checking
-    from GridCalEngine.Simulations.OPF.opf_results import OptimalPowerFlowResults
+    from GridCalEngine.Compilers.circuit_to_data import VALID_OPF_RESULTS
 
 
 def solve(nc: NumericalCircuit,
@@ -489,9 +489,15 @@ def single_island_pf(nc: NumericalCircuit,
     results = PowerFlowResults(n=nc.nbus,
                                m=nc.nbr,
                                n_hvdc=nc.nhvdc,
+                               n_gen=nc.ngen,
+                               n_batt=nc.nbatt,
+                               n_sh=nc.nshunt,
                                bus_names=nc.bus_names,
                                branch_names=nc.branch_names,
                                hvdc_names=nc.hvdc_names,
+                               gen_names=nc.generator_names,
+                               batt_names=nc.battery_names,
+                               sh_names=nc.shunt_names,
                                bus_types=bus_types)
 
     results.Sbus = solution.Scalc * nc.Sbase  # MVA
@@ -551,9 +557,9 @@ def power_flow_post_process(
         Sbus[vd] = V[vd] * np.conj(Ybus[vd, :] @ V)
 
         # Reactive power at the pv nodes
-        P = Sbus[pv].real
-        Q = (V[pv] * np.conj(Ybus[pv, :] @ V)).imag
-        Sbus[pv] = P + 1j * Q  # keep the original P injection and set the calculated reactive power
+        P_pv = Sbus[pv].real
+        Q_pv = (V[pv] * np.conj(Ybus[pv, :] @ V)).imag
+        Sbus[pv] = P_pv + 1j * Q_pv  # keep the original P injection and set the calculated reactive power for PV nodes
 
         # Branches current, loading, etc
         Vf = V[calculation_inputs.branch_data.F]
@@ -601,6 +607,42 @@ def power_flow_post_process(
     return Sfb, Stb, If, It, Vbranch, loading, losses, Sbus
 
 
+def split_reactive_power_into_devices(nc: NumericalCircuit, Qbus: Vec, results: PowerFlowResults) -> None:
+    """
+    This function splits the reactive power of the power flow solution (nbus) into reactive power per device that
+    is able to control reactive power as an injection (generators, batteries, shunts)
+    :param nc: NumericalCircuit
+    :param Qbus: Array of nodal reactive power (nbus)
+    :param results: PowerFlowResults (values are written to it)
+    :return: Nothing, the results are set in the results object
+    """
+
+    # generation
+    bus_idx_gen = nc.generator_data.get_bus_indices()
+    gen_q_share = nc.generator_data.q_share / (nc.bus_data.q_shared_total[bus_idx_gen] + 1e-20)
+
+    # batteries
+    bus_idx_bat = nc.battery_data.get_bus_indices()
+    batt_q_share = nc.battery_data.q_share / (nc.bus_data.q_shared_total[bus_idx_bat] + 1e-20)
+
+    # shunts
+    bus_idx_sh = nc.shunt_data.get_bus_indices()
+    sh_q_share = nc.shunt_data.q_share / (nc.bus_data.q_shared_total[bus_idx_sh] + 1e-20)
+
+    # Fixed injection of reactive power
+    # Zip formul: S0 + np.conj(I0 + Y0 * Vm) * Vm
+    Vm = np.abs(results.voltage)
+    Qfix = nc.bus_data.q_fixed - (nc.bus_data.ii_fixed + nc.bus_data.b_fixed * Vm) * Vm
+
+    # the remaining Q to share is the total Q computed (Qbus) minus the part that we know is fixed
+    Qvar = Qbus - Qfix
+
+    # set the results
+    results.gen_q = Qvar[bus_idx_gen] * gen_q_share
+    results.battery_q = Qvar[bus_idx_bat] * batt_q_share
+    results.shunt_q = Qvar[bus_idx_sh] * sh_q_share
+
+
 def multi_island_pf_nc(nc: NumericalCircuit,
                        options: PowerFlowOptions,
                        logger=Logger(),
@@ -621,9 +663,15 @@ def multi_island_pf_nc(nc: NumericalCircuit,
         n=nc.nbus,
         m=nc.nbr,
         n_hvdc=nc.nhvdc,
+        n_gen=nc.ngen,
+        n_batt=nc.nbatt,
+        n_sh=nc.nshunt,
         bus_names=nc.bus_data.names,
         branch_names=nc.branch_data.names,
         hvdc_names=nc.hvdc_data.names,
+        gen_names=nc.generator_names,
+        batt_names=nc.battery_names,
+        sh_names=nc.shunt_names,
         bus_types=nc.bus_data.bus_types,
     )
 
@@ -642,7 +690,6 @@ def multi_island_pf_nc(nc: NumericalCircuit,
 
     # compute islands
     islands = nc.split_into_islands(ignore_single_node_islands=options.ignore_single_node_islands)
-    results.island_number = len(islands)
 
     # initialize the all controls var
     all_controls_ok = False  # to run the first time
@@ -743,12 +790,15 @@ def multi_island_pf_nc(nc: NumericalCircuit,
     results.hvdc_loading = loading_hvdc
     results.hvdc_losses = Losses_hvdc * nc.Sbase
 
+    # do the reactive power partition and store the values
+    split_reactive_power_into_devices(nc=nc, Qbus=results.Sbus.imag, results=results)
+
     return results
 
 
 def multi_island_pf(multi_circuit: MultiCircuit,
                     options: PowerFlowOptions,
-                    opf_results: Union[OptimalPowerFlowResults, None] = None,
+                    opf_results: VALID_OPF_RESULTS | None = None,
                     t: Union[int, None] = None,
                     logger: Logger = Logger(),
                     bus_dict: Union[Dict[Bus, int], None] = None,
