@@ -139,6 +139,27 @@ class ResultsTable:
         self.cols_c, self.index_c = self.index_c, self.cols_c
         self._col_devices, self._idx_devices = self._idx_devices, self._col_devices
 
+    def sort_column(self, c: int, max_to_min: bool = True):
+        """
+
+        :param c:
+        :param max_to_min:
+        :return:
+        """
+        try:
+            sorting_arr = self.data_c[:, c].astype(float)
+        except ValueError:
+            print("Not a float column...")
+            sorting_arr = self.data_c[:, c]
+
+        if max_to_min:
+            idx = sorting_arr.argsort()[::-1]
+        else:
+            idx = sorting_arr.argsort()
+
+        self.data_c = self.data_c[idx]
+        self.index_c = self.index_c[idx]
+
     def slice_cols(self, col_idx) -> "ResultsTable":
         """
         Make column slicing
@@ -374,3 +395,38 @@ class ResultsTable:
             df.plot(ax=ax, legend=plot_legend, stacked=stacked)
         except TypeError:
             print('No numeric data to plot...')
+
+    def plot_device(self, ax=None, device_idx: int = 0, stacked=False, title: str = ""):
+        """
+        Plot the data model
+        :param ax: Matplotlib axis
+        :param device_idx: list of selected column indices
+        :param stacked: Stack plot?
+        :param title: Title of the plot
+        """
+        index, columns, data = self.get_data()
+
+        # columns = [columns[device_idx]]
+        columns = [self.title] if title == "" else [title]
+        data = data[:, device_idx]
+
+        if ax is None:
+            fig = plt.figure(figsize=(12, 6))
+            ax = fig.add_subplot(111)
+
+        if 'voltage' in self.title.lower():
+            data[data == 0] = 'nan'  # to avoid plotting the zeros
+
+        if len(columns) > 15:
+            plot_legend = False
+        else:
+            plot_legend = True
+
+        df = pd.DataFrame(data=data, index=index, columns=columns)
+        ax.set_title(self.title, fontsize=14)
+        ax.set_ylabel(self.y_label, fontsize=11)
+        ax.set_xlabel(self.x_label, fontsize=11)
+        try:
+            df.plot(ax=ax, legend=plot_legend, stacked=stacked)
+        except TypeError:
+            print('No numeric data to plot...')
