@@ -15,7 +15,7 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 from __future__ import annotations
-from typing import List, Dict, Union, Tuple, TYPE_CHECKING
+from typing import List, Dict, Union, Tuple, Set, TYPE_CHECKING
 import numpy as np
 import pandas as pd
 from scipy.sparse import lil_matrix
@@ -413,7 +413,8 @@ class TopologyProcessorInfo:
 def process_grid_topology_at(grid: MultiCircuit,
                              t_idx: Union[int, None] = None,
                              logger: Union[Logger, None] = None,
-                             debug: int = 0) -> TopologyProcessorInfo:
+                             debug: int = 0,
+                             branches_to_reduce: Set[BRANCH_TYPES] | None = None) -> TopologyProcessorInfo:
     """
     Topology processor finding the Buses that calculate a certain node-breaker topology
     This function fill the bus pointers into the grid object, and adds any new bus required for simulation
@@ -421,6 +422,7 @@ def process_grid_topology_at(grid: MultiCircuit,
     :param t_idx: Time index, None for the Snapshot
     :param logger: Logger object
     :param debug: Debug level
+    :param branches_to_reduce: Set of branches to reduce, if None the switches are reduced
     :return: TopologyProcessorInfo
     """
 
@@ -431,15 +433,15 @@ def process_grid_topology_at(grid: MultiCircuit,
     process_info = TopologyProcessorInfo()
 
     # get a list of all branches
-    all_branches = grid.get_switches() + grid.get_branches()
-    nbr = len(all_branches)
+    # all_branches = grid.get_switches() + grid.get_branches()
+    nbr = grid.get_branch_number() + grid.get_switches_number()
 
     # ------------------------------------------------------------------------------------------------------------------
     # Compose the candidate nodes (buses)
     # ------------------------------------------------------------------------------------------------------------------
 
     # find out the relevant connectivity nodes and buses from the branches
-    for k, br in enumerate(all_branches):
+    for k, br in enumerate(grid.get_all_branches_iter()):
         i = process_info.add_bus_or_cn(cn=br.cn_from, bus=br.bus_from, logger=logger, main_dev_name=br.name)
         j = process_info.add_bus_or_cn(cn=br.cn_to, bus=br.bus_to, logger=logger, main_dev_name=br.name)
         process_info.register_branch_indices(k=k, f=i, t=j)
@@ -461,20 +463,37 @@ def process_grid_topology_at(grid: MultiCircuit,
     br_active = np.empty(nbr, dtype=int)
 
     # fill matrices appropriately
-    for k, elm in enumerate(all_branches):
+    if branches_to_reduce is None:
 
-        if elm.device_type == DeviceType.SwitchDevice:
-            br_active[k] = int(elm.active) if t_idx is None else int(elm.active_prof[t_idx])
-        else:
-            # non switches form islands, because we want islands to be
-            # the set of candidates to fuse into one
+        # if no branches to reduce are provided, mark the switches for reduction
+
+        for k, elm in enumerate(grid.get_all_branches_iter()):
+
+            if elm.device_type == DeviceType.SwitchDevice:
+                br_active[k] = int(elm.active) if t_idx is None else int(elm.active_prof[t_idx])
+            else:
+                # non switches form islands, because we want islands to be
+                # the set of candidates to fuse into one
+                br_active[k] = 0
+
+            f, t = process_info.get_branch_registered_indices(k)
+
+            if br_active[k] and f is not None and t is not None:  # avoid adding zeros
+                Cf[k, f] = br_active[k]
+                Ct[k, t] = br_active[k]
+    else:
+
+        # if there were branches to reduce provided, mark them for reduction
+
+        br_idx_dict = {elm: k for k, elm in enumerate(grid.get_all_branches_iter())}
+        for elm in branches_to_reduce:
+            k = br_idx_dict[elm]
             br_active[k] = 0
+            f, t = process_info.get_branch_registered_indices(k)
 
-        f, t = process_info.get_branch_registered_indices(k)
-
-        if br_active[k] and f is not None and t is not None:  # avoid adding zeros
-            Cf[k, f] = br_active[k]
-            Ct[k, t] = br_active[k]
+            if br_active[k] and f is not None and t is not None:  # avoid adding zeros
+                Cf[k, f] = br_active[k]
+                Ct[k, t] = br_active[k]
 
     # ------------------------------------------------------------------------------------------------------------------
     # Compose the adjacency matrix from the connectivity information
@@ -486,7 +505,7 @@ def process_grid_topology_at(grid: MultiCircuit,
 
     if debug >= 2:
         candidate_names = process_info.get_candidate_names()
-        br_names = [br.name for br in all_branches]
+        br_names = [br.name for br in grid.get_all_branches_iter()]
         C = Cf + Ct
         df = pd.DataFrame(data=C.toarray(), columns=candidate_names, index=br_names)
         print(df.replace(to_replace=0.0, value="-"))
@@ -524,7 +543,7 @@ def process_grid_topology_at(grid: MultiCircuit,
                 logger.add_info("Bus added to grid", device=bus_device.name)
 
     # map the buses to the branches from their connectivity nodes
-    for i, elm in enumerate(all_branches):
+    for i, elm in enumerate(grid.get_all_branches_iter()):
         if elm.cn_from is not None:
             elm.set_bus_from_at(t_idx=t_idx, val=process_info.get_final_bus(elm.cn_from))
 
