@@ -1,5 +1,5 @@
 # GridCal
-# Copyright (C) 2015 - 2023 Santiago Peñate Vera
+# Copyright (C) 2015 - 2024 Santiago Peñate Vera
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -15,13 +15,13 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-from typing import List, Any, Dict, Union
-from enum import Enum
+from typing import List, Any, Dict, Union, Tuple
 import pandas as pd
 import numpy as np
 import datetime
 import nptyping as npt
 from scipy.sparse import csc_matrix, csr_matrix
+from GridCalEngine.enumerations import TimeGrouping, LogSeverity
 
 IntList = List[int]
 Numeric = Union[int, float, bool, complex]
@@ -42,383 +42,12 @@ CscMat = csc_matrix
 CsrMat = csr_matrix
 
 
-class BusMode(Enum):
-    """
-    Emumetarion of bus modes
-    """
-    PQ = 1
-    PV = 2
-    Slack = 3
-    NONE = 4
-    STO_DISPATCH = 5  # Storage dispatch, in practice it is the same as REF
-    PVB = 6
-
-    def __str__(self):
-        return self.value
-
-    def __repr__(self):
-        return str(self)
-
-    @staticmethod
-    def argparse(s):
-        """
-
-        :param s:
-        :return:
-        """
-        try:
-            return BusMode[s]
-        except KeyError:
-            return s
-
-
-class ExternalGridMode(Enum):
-    """
-    Modes of operation of external grids
-    """
-    PQ = "PQ"
-    PV = "PV"
-    VD = "VD"
-
-    def __str__(self):
-        return self.value
-
-    def __repr__(self):
-        return str(self)
-
-    @staticmethod
-    def argparse(s):
-        """
-
-        :param s:
-        :return:
-        """
-        try:
-            return ExternalGridMode[s]
-        except KeyError:
-            return s
-
-
-class InvestmentEvaluationMethod(Enum):
-    """
-    Investment evaluation methods
-    """
-    Independent = "Independent"
-    Hyperopt = "Hyperopt"
-    MVRSM = "MVRSM"
-
-    def __str__(self):
-        return self.value
-
-    def __repr__(self):
-        return str(self)
-
-    @staticmethod
-    def argparse(s):
-        """
-
-        :param s:
-        :return:
-        """
-        try:
-            return InvestmentEvaluationMethod[s]
-        except KeyError:
-            return s
-
-
-class BranchImpedanceMode(Enum):
-    """
-    Enumeration of branch impedance modes
-    """
-    Specified = 0
-    Upper = 1
-    Lower = 2
-
-    def __str__(self):
-        return self.value
-
-    def __repr__(self):
-        return str(self)
-
-    @staticmethod
-    def argparse(s):
-        """
-
-        :param s:
-        :return:
-        """
-        try:
-            return BranchImpedanceMode[s]
-        except KeyError:
-            return s
-
-
-class SolverType(Enum):
-    """
-    Refer to the :ref:`Power Flow section<power_flow>` for details about the different
-    algorithms supported by **GridCal**.
-    """
-
-    NR = 'Newton Raphson'
-    NRD = 'Newton Raphson Decoupled'
-    NRFD_XB = 'Fast decoupled XB'
-    NRFD_BX = 'Fast decoupled BX'
-    GAUSS = 'Gauss-Seidel'
-    DC = 'Linear DC'
-    HELM = 'Holomorphic Embedding'
-    ZBUS = 'Z-Gauss-Seidel'
-    IWAMOTO = 'Iwamoto-Newton-Raphson'
-    CONTINUATION_NR = 'Continuation-Newton-Raphson'
-    HELMZ = 'HELM-Z'
-    LM = 'Levenberg-Marquardt'
-    FASTDECOUPLED = 'Fast decoupled'
-    LACPF = 'Linear AC'
-    DC_OPF = 'Linear DC OPF'
-    AC_OPF = 'Nonlinear AC OPF'
-    Simple_OPF = 'Simple dispatch'
-    Proportional_OPF = 'Proportional OPF'
-    NRI = 'Newton-Raphson in current'
-    DYCORS_OPF = 'DYCORS OPF'
-    GA_OPF = 'Genetic Algorithm OPF'
-    NELDER_MEAD_OPF = 'Nelder Mead OPF'
-    BFS = 'Backwards-Forward substitution'
-    BFS_linear = 'Backwards-Forward substitution (linear)'
-    Constant_Impedance_linear = 'Constant impedance linear'
-    NoSolver = 'No Solver'
-
-    def __str__(self):
-        return self.value
-
-    def __repr__(self):
-        return str(self)
-
-    @staticmethod
-    def argparse(s):
-        """
-
-        :param s:
-        :return:
-        """
-        try:
-            return SolverType[s]
-        except KeyError:
-            return s
-
-
-class ReactivePowerControlMode(Enum):
-    """
-    The :ref:`ReactivePowerControlMode<q_control>` offers 3 modes to control how
-    :ref:`Generator<generator>` objects supply reactive power:
-
-    **NoControl**: In this mode, the :ref:`generators<generator>` don't try to regulate
-    the voltage at their :ref:`bus<bus>`.
-
-    **Direct**: In this mode, the :ref:`generators<generator>` try to regulate the
-    voltage at their :ref:`bus<bus>`. **GridCal** does so by applying the following
-    algorithm in an outer control loop. For grids with numerous
-    :ref:`generators<generator>` tied to the same system, for example wind farms, this
-    control method sometimes fails with some :ref:`generators<generator>` not trying
-    hard enough*. In this case, the simulation converges but the voltage controlled
-    :ref:`buses<bus>` do not reach their target voltage, while their
-    :ref:`generator(s)<generator>` haven't reached their reactive power limit. In this
-    case, the slower **Iterative** control mode may be used (see below).
-
-        ON PV-PQ BUS TYPE SWITCHING LOGIC IN POWER FLOW COMPUTATION
-        Jinquan Zhao
-
-        1) Bus i is a PQ bus in the previous iteration and its
-           reactive power was fixed at its lower limit:
-
-            If its voltage magnitude Vi >= Viset, then
-
-                it is still a PQ bus at current iteration and set Qi = Qimin .
-
-                If Vi < Viset , then
-
-                    compare Qi with the upper and lower limits.
-
-                    If Qi >= Qimax , then
-                        it is still a PQ bus but set Qi = Qimax .
-                    If Qi <= Qimin , then
-                        it is still a PQ bus and set Qi = Qimin .
-                    If Qimin < Qi < Qi max , then
-                        it is switched to PV bus, set Vinew = Viset.
-
-        2) Bus i is a PQ bus in the previous iteration and
-           its reactive power was fixed at its upper limit:
-
-            If its voltage magnitude Vi <= Viset , then:
-                bus i still a PQ bus and set Q i = Q i max.
-
-                If Vi > Viset , then
-
-                    Compare between Qi and its upper/lower limits
-
-                    If Qi >= Qimax , then
-                        it is still a PQ bus and set Q i = Qimax .
-                    If Qi <= Qimin , then
-                        it is still a PQ bus but let Qi = Qimin in current iteration.
-                    If Qimin < Qi < Qimax , then
-                        it is switched to PV bus and set Vinew = Viset
-
-        3) Bus i is a PV bus in the previous iteration.
-
-            Compare Q i with its upper and lower limits.
-
-            If Qi >= Qimax , then
-                it is switched to PQ and set Qi = Qimax .
-            If Qi <= Qimin , then
-                it is switched to PQ and set Qi = Qimin .
-            If Qi min < Qi < Qimax , then
-                it is still a PV bus.
-
-    **Iterative**: As mentioned above, the **Direct** control mode may not yield
-    satisfying results in some isolated cases. The **Direct** control mode tries to
-    jump to the final solution in a single or few iterations, but in grids where a
-    significant change in reactive power at one :ref:`generator<generator>` has a
-    significant impact on other :ref:`generators<generator>`, additional iterations may
-    be required to reach a satisfying solution.
-
-    Instead of trying to jump to the final solution, the **Iterative** mode raises or
-    lowers each :ref:`generator's<generator>` reactive power incrementally. The
-    increment is determined using a logistic function based on the difference between
-    the current :ref:`bus<bus>` voltage its target voltage. The steepness factor
-    :code:`k` of the logistic function was determined through trial and error, with the
-    intent of reducing the number of iterations while avoiding instability. Other
-    values may be specified in :ref:`PowerFlowOptions<pf_options>`.
-
-    The :math:`Q_{Increment}` in per unit is determined by:
-
-    .. math::
-
-        Q_{Increment} = 2 * \\left[\\frac{1}{1 + e^{-k|V_2 - V_1|}}-0.5\\right]
-
-    Where:
-
-        k = 30 (by default)
-
-    """
-    NoControl = "NoControl"
-    Direct = "Direct"
-    Iterative = "Iterative"
-
-
-class TapsControlMode(Enum):
-    """
-    The :ref:`TapsControlMode<taps_control>` offers 3 modes to control how
-    :ref:`transformers<transformer>`' :ref:`tap changer<tap_changer>` regulate
-    voltage on their regulated :ref:`bus<bus>`:
-
-    **NoControl**: In this mode, the :ref:`transformers<transformer>` don't try to
-    regulate the voltage at their :ref:`bus<bus>`.
-
-    **Direct**: In this mode, the :ref:`transformers<transformer>` try to regulate
-    the voltage at their bus. **GridCal** does so by jumping straight to the tap that
-    corresponds to the desired transformation ratio, or the highest or lowest tap if
-    the desired ratio is outside of the tap range.
-
-    This behavior may fail in certain cases, especially if both the
-    :ref:`TapControlMode<taps_control>` and :ref:`ReactivePowerControlMode<q_control>`
-    are set to **Direct**. In this case, the simulation converges but the voltage
-    controlled :ref:`buses<bus>` do not reach their target voltage, while their
-    :ref:`generator(s)<generator>` haven't reached their reactive power limit. When
-    this happens, the slower **Iterative** control mode may be used (see below).
-
-    **Iterative**: As mentioned above, the **Direct** control mode may not yield
-    satisfying results in some isolated cases. The **Direct** control mode tries to
-    jump to the final solution in a single or few iterations, but in grids where a
-    significant change of tap at one :ref:`transformer<transformer>` has a
-    significant impact on other :ref:`transformers<transformer>`, additional
-    iterations may be required to reach a satisfying solution.
-
-    Instead of trying to jump to the final solution, the **Iterative** mode raises or
-    lowers each :ref:`transformer's<transformer>` tap incrementally.
-    """
-
-    NoControl = "NoControl"
-    Direct = "Direct"
-    Iterative = "Iterative"
-
-    def __str__(self):
-        return self.value
-
-    def __repr__(self):
-        return str(self)
-
-    @staticmethod
-    def argparse(s):
-        """
-
-        :param s:
-        :return:
-        """
-        try:
-            return TapsControlMode[s]
-        except KeyError:
-            return s
-
-
-class SyncIssueType(Enum):
-    """
-    Sync issues enumeration
-    """
-    Added = 'Added'
-    Deleted = 'Deleted'
-    Conflict = 'Conflict'
-
-    def __str__(self):
-        return self.value
-
-    def __repr__(self):
-        return str(self)
-
-    @staticmethod
-    def argparse(s):
-        """
-
-        :param s:
-        :return:
-        """
-        try:
-            return SyncIssueType[s]
-        except KeyError:
-            return s
-
-
-class EngineType(Enum):
-    """
-    Available engines enumeration
-    """
-    GridCal = 'GridCal'
-    Bentayga = 'Bentayga'
-    NewtonPA = 'Newton Power Analytics'
-    PGM = 'Power Grid Model'
-
-    def __str__(self):
-        return self.value
-
-    def __repr__(self):
-        return str(self)
-
-    @staticmethod
-    def argparse(s):
-        """
-
-        :param s:
-        :return:
-        """
-        try:
-            return EngineType[s]
-        except KeyError:
-            return s
-
-
 class CDF:
     """
     Inverse Cumulative density function of a given array of data
     """
 
-    def __init__(self, data):
+    def __init__(self, data: Vec):
         """
         Constructor
         @param data: Array (list or numpy array)
@@ -503,7 +132,7 @@ class CDF:
         else:
             return np.interp(pt, self.prob, self.arr)
 
-    def get_at(self, prob):
+    def get_at(self, prob: float):
         """
         Samples a number of uniform distributed points and
         returns the corresponding probability values given the CDF.
@@ -517,11 +146,13 @@ class CDF:
         else:
             return np.interp(prob, self.prob, self.arr)
 
-    def plot(self, plt, LINEWIDTH, ax=None):
+    def plot(self, plt, LINEWIDTH: int, ax=None):
         """
         Plots the CFD
-        @param ax: MatPlotLib axis to plot into
-        @return:
+        :param plt: MatPlotLib plt module
+        :param LINEWIDTH: line width in pixels
+        :param ax: MatPlotLib axis to plot into
+        :return:
         """
         if ax is None:
             fig = plt.figure()
@@ -529,121 +160,6 @@ class CDF:
         ax.plot(self.prob, self.arr, linewidth=LINEWIDTH)
         ax.set_xlabel('$p(x)$')
         ax.set_ylabel('$x$')
-        # ax.plot(self.norm_points, self.values, 'x')
-
-
-class MIPSolvers(Enum):
-    """
-    MIP solvers enumeration
-    """
-    GLOP = "GLOP"
-    CBC = 'CBC'
-    HIGHS = 'HIGHS'
-    SCIP = 'SCIP'
-    CPLEX = 'CPLEX'
-    GUROBI = 'GUROBI'
-    XPRESS = 'XPRESS'
-
-    def __str__(self):
-        return self.value
-
-    def __repr__(self):
-        return str(self)
-
-    @staticmethod
-    def argparse(s):
-        """
-
-        :param s:
-        :return:
-        """
-        try:
-            return MIPSolvers[s]
-        except KeyError:
-            return s
-
-
-class TimeGrouping(Enum):
-    """
-    Time groupings enumeration
-    """
-    NoGrouping = 'No grouping'
-    Monthly = 'Monthly'
-    Weekly = 'Weekly'
-    Daily = 'Daily'
-    Hourly = 'Hourly'
-
-    def __str__(self):
-        return self.value
-
-    def __repr__(self):
-        return str(self)
-
-    @staticmethod
-    def argparse(s):
-        """
-
-        :param s:
-        :return:
-        """
-        try:
-            return TimeGrouping[s]
-        except KeyError:
-            return s
-
-
-class ZonalGrouping(Enum):
-    """
-    Zonal groupings enumeration
-    """
-    NoGrouping = 'No grouping'
-    Area = 'Area'
-    All = 'All (copper plate)'
-
-    def __str__(self):
-        return self.value
-
-    def __repr__(self):
-        return str(self)
-
-    @staticmethod
-    def argparse(s):
-        """
-
-        :param s:
-        :return:
-        """
-        try:
-            return ZonalGrouping[s]
-        except KeyError:
-            return s
-
-
-class ContingencyEngine(Enum):
-    """
-    Enumeratio of contingency calculation engines
-    """
-    PowerFlow = 'Power flow'
-    HELM = 'HELM'
-    PTDF = 'PTDF'
-
-    def __str__(self):
-        return self.value
-
-    def __repr__(self):
-        return str(self)
-
-    @staticmethod
-    def argparse(s):
-        """
-
-        :param s:
-        :return:
-        """
-        try:
-            return ZonalGrouping[s]
-        except KeyError:
-            return s
 
 
 def classify_by_hour(t: pd.DatetimeIndex) -> List[List[int]]:
@@ -658,7 +174,7 @@ def classify_by_hour(t: pd.DatetimeIndex) -> List[List[int]]:
     offset = t[0].hour * t[0].dayofyear
     mx = t[n - 1].hour * t[n - 1].dayofyear
 
-    arr = list()
+    arr: List[List[int]] = list()
 
     for i in range(mx - offset + 1):
         arr.append(list())
@@ -670,7 +186,7 @@ def classify_by_hour(t: pd.DatetimeIndex) -> List[List[int]]:
     return arr
 
 
-def classify_by_day(t: pd.DatetimeIndex) -> list[list[Any]]:
+def classify_by_day(t: pd.DatetimeIndex) -> list[list[int]]:
     """
     Passes an array of TimeStamps to an array of arrays of indices
     classified by day of the year
@@ -682,7 +198,7 @@ def classify_by_day(t: pd.DatetimeIndex) -> list[list[Any]]:
     offset = t[0].dayofyear
     mx = t[n - 1].dayofyear
 
-    arr = list()
+    arr: list[list[int]] = list()
 
     for i in range(mx - offset + 1):
         arr.append(list())
@@ -701,7 +217,7 @@ def get_time_groups(t_array: pd.DatetimeIndex, grouping: TimeGrouping) -> List[i
     :param grouping: TimeGrouping value
     :return: list of indices that determine the partitions
     """
-    groups = list()
+    groups: List[int] = list()
 
     last = -1
 
@@ -736,53 +252,51 @@ def get_time_groups(t_array: pd.DatetimeIndex, grouping: TimeGrouping) -> List[i
     return groups
 
 
-class LogSeverity(Enum):
-    """
-    Enumeration of logs severities
-    """
-    Error = 'Error'
-    Warning = 'Warning'
-    Information = 'Information'
-    Divergence = 'Divergence'
-
-    def __str__(self):
-        return self.value
-
-    def __repr__(self):
-        return str(self)
-
-    @staticmethod
-    def argparse(s):
-        """
-
-        :param s:
-        :return:
-        """
-        try:
-            return LogSeverity[s]
-        except KeyError:
-            return s
-
-
 class LogEntry:
     """
     Logger entry
     """
 
-    def __init__(self, msg="", severity: LogSeverity = LogSeverity.Information, device="", value="", expected_value=""):
-        self.time = "{date:%H:%M:%S}".format(date=datetime.datetime.now())  # might use %Y/%m/%d %H:%M:%S
+    def __init__(self,
+                 time: Union[str, None] = None,
+                 msg="",
+                 severity: LogSeverity = LogSeverity.Information,
+                 device="",
+                 value="",
+                 expected_value="",
+                 device_class="",
+                 device_property="",
+                 object_value=None,
+                 expected_object_value=None):
+        if time is None:
+            self.time = "{date:%H:%M:%S}".format(date=datetime.datetime.now())  # might use %Y/%m/%d %H:%M:%S
+        else:
+            self.time = time
         self.msg = str(msg)
         self.severity = severity
         self.device = device
+        self.device_class = device_class
+        self.device_property = device_property
         self.value = value
         self.expected_value = str(expected_value)
+        self.object_value = object_value
+        self.expected_object_value = expected_object_value
 
     def to_list(self) -> List[Any]:
         """
         Get list representation of this entry
         :return:
         """
-        return [self.time, self.severity.value, self.msg, self.device, self.value, self.expected_value]
+        return [self.time, self.severity.value, self.msg,
+                self.device_class, self.device_property, self.device,
+                self.value, self.expected_value]
+
+    def to_list_reduced(self) -> List[Any]:
+        """
+        Get list representation of this entry
+        :return:
+        """
+        return [self.time, self.device_class, self.device_property,  self.device, self.value, self.expected_value]
 
     def __str__(self):
         return "{0} {1}: {2} {3} {4} {5}".format(self.time,
@@ -798,7 +312,7 @@ class Logger:
     Logger class
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
 
         self.entries: List[LogEntry] = list()
 
@@ -826,7 +340,8 @@ class Logger:
         """
         return len(self.entries) > 0
 
-    def add_info(self, msg: str, device="", value="", expected_value="", device_class='', comment='', device_property=''):
+    def add_info(self, msg: str, device="", value="", expected_value="", device_class='', comment='',
+                 device_property='', object_value=None, expected_object_value=None):
         """
         Add info entry
         :param msg:
@@ -835,11 +350,23 @@ class Logger:
         :param expected_value:
         :param device_class:
         :param comment:
+        :param device_property:
+        :param object_value:
+        :param expected_object_value:
         :return:
         """
-        self.entries.append(LogEntry(msg, LogSeverity.Information, device, str(value), str(expected_value)))
+        self.entries.append(LogEntry(msg=str(msg),
+                                     severity=LogSeverity.Information,
+                                     device=str(device),
+                                     value=str(value),
+                                     expected_value=str(expected_value),
+                                     device_class=str(device_class),
+                                     device_property=str(device_property),
+                                     object_value=str(object_value),
+                                     expected_object_value=str(expected_object_value)))
 
-    def add_warning(self, msg, device="", value="", expected_value="", device_class='', comment='', device_property=''):
+    def add_warning(self, msg: str, device="", value="", expected_value="", device_class='', comment='',
+                    device_property='', object_value=None, expected_object_value=None):
         """
         Add warning entry
         :param msg:
@@ -848,11 +375,23 @@ class Logger:
         :param expected_value:
         :param device_class:
         :param comment:
+        :param device_property:
+        :param object_value:
+        :param expected_object_value:
         :return:
         """
-        self.entries.append(LogEntry(msg, LogSeverity.Warning, device, str(value), str(expected_value)))
+        self.entries.append(LogEntry(msg=str(msg),
+                                     severity=LogSeverity.Warning,
+                                     device=str(device),
+                                     value=str(value),
+                                     expected_value=str(expected_value),
+                                     device_class=str(device_class),
+                                     device_property=str(device_property),
+                                     object_value=str(object_value),
+                                     expected_object_value=str(expected_object_value)))
 
-    def add_error(self, msg, device="", value="", expected_value="", device_class='', comment='', device_property=''):
+    def add_error(self, msg: str, device="", value="", expected_value="", device_class='', comment='',
+                  device_property='', object_value=None, expected_object_value=None):
         """
         Add error entry
         :param msg:
@@ -861,9 +400,20 @@ class Logger:
         :param expected_value:
         :param device_class:
         :param comment:
+        :param device_property:
+        :param object_value:
+        :param expected_object_value:
         :return:
         """
-        self.entries.append(LogEntry(msg, LogSeverity.Error, device, str(value), str(expected_value)))
+        self.entries.append(LogEntry(msg=str(msg),
+                                     severity=LogSeverity.Error,
+                                     device=str(device),
+                                     value=str(value),
+                                     expected_value=str(expected_value),
+                                     device_class=str(device_class),
+                                     device_property=str(device_property),
+                                     object_value=str(object_value),
+                                     expected_object_value=str(expected_object_value)))
 
     def add_divergence(self, msg, device="", value=0, expected_value=0, tol=1e-6):
         """
@@ -877,27 +427,52 @@ class Logger:
         """
 
         if abs(value - expected_value) > tol:
-            self.entries.append(LogEntry(msg, LogSeverity.Divergence, device, str(value), str(expected_value)))
+            self.entries.append(LogEntry(msg=str(msg),
+                                         severity=LogSeverity.Divergence,
+                                         device=str(device),
+                                         value=str(value),
+                                         expected_value=str(expected_value),
+                                         device_class="",
+                                         device_property="",
+                                         object_value=None,
+                                         expected_object_value=None))
 
-    def add(self, msg, severity: LogSeverity = LogSeverity.Error, device="", value="", expected_value=""):
+    def add(self, msg: str, severity: LogSeverity = LogSeverity.Error, device="", value="", expected_value="",
+            device_class='', comment='', device_property='', object_value=None, expected_object_value=None):
         """
         Add general entry
         :param msg:
         :param severity:
         :param device:
         :param value:
-        :param expected_value
+        :param expected_value:
+        :param device_class:
+        :param comment:
+        :param device_property:
+        :param object_value:
+        :param expected_object_value:
         :return:
         """
-        self.entries.append(LogEntry(msg, severity, device, str(value), str(expected_value)))
+        # self.entries.append(LogEntry(msg, severity, device, str(value), str(expected_value)))
+        self.entries.append(LogEntry(msg=str(msg),
+                                     severity=severity,
+                                     device=str(device),
+                                     value=str(value),
+                                     expected_value=str(expected_value),
+                                     device_class=str(device_class),
+                                     device_property=str(device_property),
+                                     object_value=str(object_value),
+                                     expected_object_value=str(expected_object_value)))
 
-    def to_dict(self):
+    def to_dict(self) -> Union[Dict[str, Dict[str, List[Tuple[str, str, str, str]]]], Dict[str, Dict[str, List[List[str]]]]]:
         """
         Get the logs sorted by severity and message
         :return: Dictionary[Dictionary[List[time, device, value, expected value]]]
         """
         by_severity = dict()
 
+        hdr = ['Time', 'Class', 'Property', 'Device', 'Value', 'Expected value']
+
         for e in self.entries:
 
             if e.severity.value not in by_severity.keys():
@@ -907,23 +482,40 @@ class Logger:
 
             if e.msg in by_msg.keys():
                 # add msg to existing msg list
-                by_msg[e.msg].append((e.time, e.device, e.value, e.expected_value))
+                by_msg[e.msg].append(e.to_list_reduced())
             else:
                 # add msg entry for the first time
-                by_msg[e.msg] = [(e.time, e.device, e.value, e.expected_value)]
+                by_msg[e.msg] = [e.to_list_reduced()]
 
         return by_severity
 
-    def to_df(self):
+    def to_df(self) -> pd.DataFrame:
         """
         Get DataFrame
-        :return:
+        :return: DataFrame
         """
         data = [e.to_list() for e in self.entries]
-        df = pd.DataFrame(data=data, columns=['Time', 'Severity', 'Message', 'Device', 'Value', 'Expected value'])
+        df = pd.DataFrame(data=data, columns=['Time', 'Severity', 'Message', 'Class',
+                                              'Property', 'Device', 'Value', 'Expected value'])
         df.set_index('Time', inplace=True)
         return df
 
+    def parse_df(self, df: pd.DataFrame):
+        """
+        Parse DataFrame
+        :param df: DataFrame
+        """
+        for i, row in df.iterrows():
+            self.entries.append(LogEntry(msg=str(row["Message"]),
+                                         severity=LogSeverity(row["Severity"]),
+                                         device=str(row["Device"]),
+                                         value=str(row["Value"]),
+                                         expected_value=str(row["Expected value"]),
+                                         device_class=str(row["Class"]),
+                                         device_property=str(row["Property"]),
+                                         object_value="",
+                                         expected_object_value=""))
+
     def to_csv(self, fname):
         """
         Save to CSV
@@ -938,6 +530,14 @@ class Logger:
         """
         self.to_df().to_excel(fname)
 
+    def print(self, title: str = "") -> None:
+        """
+        Print the logs
+        """
+        if title != "":
+            print(title)
+        print(self.to_df())
+
     def __str__(self):
 
         val = ''
@@ -973,7 +573,7 @@ class Logger:
             self.entries += other.entries
         return self
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self.entries)
 
     def size(self) -> int:
@@ -983,13 +583,50 @@ class Logger:
         """
         return len(self.entries)
 
+    def count_type(self, severity: LogSeverity) -> int:
+        """
+        Count the number of entries of a certain severity
+        :param severity: LogSeverity
+        :return: number of occurences
+        """
+        c = 0
+        for entry in self.entries:
+            if entry.severity == severity:
+                c += 1
+
+        return c
+
+    def info_count(self) -> int:
+        """
+        Count the number of information occurences
+        :return:
+        """
+        return self.count_type(LogSeverity.Information)
+
+    def warning_count(self) -> int:
+        """
+        Count number of warnings
+        :return:
+        """
+        return self.count_type(LogSeverity.Warning)
+
+    def error_count(self) -> int:
+        """
+        Count number of errors
+        :return:
+        """
+        return self.count_type(LogSeverity.Error)
+
 
 class ConvergenceReport:
     """
     Convergence report
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
+        """
+        Constructor
+        """
         self.methods_ = list()
         self.converged_ = list()
         self.error_ = list()
@@ -1012,7 +649,7 @@ class ConvergenceReport:
         self.elapsed_.append(elapsed)
         self.iterations_.append(iterations)
 
-    def converged(self):
+    def converged(self) -> bool:
         """
 
         :return:
@@ -1022,7 +659,7 @@ class ConvergenceReport:
         else:
             return False
 
-    def error(self):
+    def error(self) -> float:
         """
 
         :return:
@@ -1030,9 +667,9 @@ class ConvergenceReport:
         if len(self.error_) > 0:
             return self.error_[-1]
         else:
-            return 0
+            return 0.0
 
-    def elapsed(self):
+    def elapsed(self) -> float:
         """
 
         :return:
@@ -1040,9 +677,9 @@ class ConvergenceReport:
         if len(self.elapsed_) > 0:
             return self.elapsed_[-1]
         else:
-            return 0
+            return 0.0
 
-    def to_dataframe(self):
+    def to_dataframe(self) -> pd.DataFrame:
         """
 
         :return:
@@ -1053,9 +690,7 @@ class ConvergenceReport:
                 'Elapsed (s)': self.elapsed_,
                 'Iterations': self.iterations_}
 
-        df = pd.DataFrame(data)
-
-        return df
+        return pd.DataFrame(data)
 
 
 def get_list_dim(a: List[Any]) -> int:
@@ -1064,11 +699,11 @@ def get_list_dim(a: List[Any]) -> int:
     :param a: some List
     :return: Dimensions
     """
-    if not type(a) == list:
+    if not isinstance(a, list):
         return 0
     else:
         if len(a) > 0:
-            if type(a[0]) == list:
+            if isinstance(a[0], list):
                 return 2
             else:
                 return 1
@@ -1115,7 +750,7 @@ class CompressedJsonStruct:
         :param dta: list of lists
         :return: Nothing
         """
-        if type(dta) == list:
+        if isinstance(dta, list):
             if len(dta) > 0:
 
                 dim = get_list_dim(dta)
