@@ -437,8 +437,6 @@ class MIPSolvers(Enum):
     """
     MIP solvers enumeration
     """
-    GLOP = "GLOP"
-    CBC = 'CBC'
     HIGHS = 'HIGHS'
     SCIP = 'SCIP'
     CPLEX = 'CPLEX'
@@ -461,7 +459,7 @@ class MIPSolvers(Enum):
         try:
             return MIPSolvers[s]
         except KeyError:
-            return s
+            return MIPSolvers.HIGHS
 
 
 class TimeGrouping(Enum):
@@ -974,6 +972,7 @@ class DeviceType(Enum):
     GeneratorDevice = 'Generator'
     StaticGeneratorDevice = 'Static Generator'
     ShuntDevice = 'Shunt'
+    ShuntLikeDevice = 'Shunt like devices'
     UpfcDevice = 'UPFC'  # unified power flow controller
     ExternalGridDevice = 'External grid'
     LoadLikeDevice = 'Load like'
@@ -991,6 +990,7 @@ class DeviceType(Enum):
     SequenceLineDevice = 'Sequence line'
     UnderGroundLineDevice = 'Underground line'
     OverheadLineTypeDevice = 'Tower'
+    AnyLineTemplateDevice = "Any line template"
     TransformerTypeDevice = 'Transformer type'
     SwitchDevice = 'Switch'
 
@@ -1000,11 +1000,12 @@ class DeviceType(Enum):
     AreaDevice = 'Area'
     ZoneDevice = 'Zone'
     CountryDevice = 'Country'
-    CommunityDevice = 'Comunity'
+    CommunityDevice = 'Community'
     RegionDevice = 'Region'
     MunicipalityDevice = 'Municipality'
     BusBarDevice = 'BusBar'
     VoltageLevelDevice = 'Voltage level'
+    VoltageLevelTemplate = 'Voltage level template'
 
     Technology = 'Technology'
     TechnologyGroup = 'Technology Group'
@@ -1013,6 +1014,9 @@ class DeviceType(Enum):
     ContingencyDevice = 'Contingency'
     ContingencyGroupDevice = 'Contingency Group'
 
+    RemedialActionDevice = 'Remedial action'
+    RemedialActionGroupDevice = 'Remedial action Group'
+
     InvestmentDevice = 'Investment'
     InvestmentsGroupDevice = 'Investments Group'
 
@@ -1037,6 +1041,8 @@ class DeviceType(Enum):
 
     ModellingAuthority = "Modelling Authority"
 
+    FacilityDevice = "Facility"
+
     SimulationOptionsDevice = "SimulationOptionsDevice"
 
     InterAggregationInfo = "InterAggregationInfo"
@@ -1427,6 +1433,8 @@ class ResultTypes(Enum):
     BusVoltagePolar = 'Voltage (polar)'
     BusActivePower = 'P: Active power'
     BusReactivePower = 'Q: Reactive power'
+    BusActivePowerIncrement = "ΔP: Active power increment"
+
     BranchPower = 'Sf: Power'
     BranchActivePowerFrom = 'Pf: Active power "from"'
     BranchReactivePowerFrom = 'Qf: Reactive power "from"'
@@ -1483,12 +1491,17 @@ class ResultTypes(Enum):
     LoadShedding = 'Load shedding'
     GeneratorShedding = 'Generator shedding'
     GeneratorPower = 'Generator power'
+    GeneratorReactivePower = 'Generator reactive power'
     GeneratorCost = 'Generator cost'
     GeneratorFuels = 'Generator fuels'
     GeneratorEmissions = 'Generator emissions'
     GeneratorProducing = 'Generator producing'
     GeneratorStartingUp = 'Generator starting up'
     GeneratorShuttingDown = 'Generator shutting down'
+
+    BatteryReactivePower = 'Battery reactive power'
+    ShuntReactivePower = 'Shunt reactive power'
+
     BusVoltagePolarPlot = 'Voltage plot'
     BusNodalCapacity = "Nodal capacity"
 
@@ -1502,6 +1515,8 @@ class ResultTypes(Enum):
     LossesPerArea = 'Losses per area'
     ActivePowerFlowPerArea = 'Active power flow per area'
     LossesPerGenPerArea = 'Losses per generation unit in area'
+    InterSpaceBranchPower = "Inter-space branch power"
+    InterSpaceBranchLoading = "Inter-space branch loading"
 
     SystemFuel = 'System fuel consumption'
     SystemEmissions = 'System emissions'
@@ -1663,6 +1678,7 @@ class ResultTypes(Enum):
     GeneratorResults = 'Generators'
     LoadResults = 'Loads'
     BatteryResults = 'Batteries'
+    ShuntResults = 'Shunt like devices'
     StatisticResults = 'Statistics'
 
     # fluid
@@ -1824,6 +1840,9 @@ class ContingencyFilteringMethods(Enum):
 
 
 class Colormaps(Enum):
+    """
+    Available colormaps
+    """
     GridCal = 'GridCal'
     TSO = 'TSO'  # -1, 1
     TSO2 = 'TSO 2'  # -1, 1
@@ -1852,3 +1871,44 @@ class Colormaps(Enum):
             return Colormaps[s]
         except KeyError:
             return s
+
+
+class SubstationTypes(Enum):
+    """
+    Types of substation types
+    """
+    SingleBar = 'Single bar'
+    SingleBarWithBypass = 'Single bar with bypass'
+    SingleBarWithSplitter = 'Single bar with splitter'
+    DoubleBar = "Double bar"
+    DoubleBarWithBypass = "Double bar with bypass"
+    DoubleBarWithTransference = "Double bar with transference bar"
+    DoubleBarDuplex = "Double bar duplex"
+    Ring = 'Ring'
+    BreakerAndAHalf = 'Breaker and a half'
+
+    def __str__(self) -> str:
+        return str(self.value)
+
+    def __repr__(self):
+        return str(self)
+
+    @staticmethod
+    def argparse(s):
+        """
+
+        :param s:
+        :return:
+        """
+        try:
+            return SubstationTypes[s]
+        except KeyError:
+            return s
+
+    @classmethod
+    def list(cls):
+        """
+
+        :return:
+        """
+        return list(map(lambda c: c.value, cls))
